SystemOrganization addCategory: #'OmniBrowser-Actors'!
SystemOrganization addCategory: #'OmniBrowser-Kernel'!
SystemOrganization addCategory: #'OmniBrowser-Morphic'!
SystemOrganization addCategory: #'OmniBrowser-Nodes'!
SystemOrganization addCategory: #'OmniBrowser-Notifications'!
SystemOrganization addCategory: #'OmniBrowser-Panels'!
SystemOrganization addCategory: #'OmniBrowser-Tests'!
SystemOrganization addCategory: #'OmniBrowser-Utilities'!

TextMorphEditor subclass: #OBTextMorphEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBTextMorphEditor methodsFor: 'actions' stamp: 'cwp 10/31/2004 00:26'!
sendersOfIt
	"Open a senders browser on the selected selector"

	| selector |
	self lineSelectAndEmptyCheck: [^ self].
	(selector _ self selectedSelector) == nil ifTrue: [^ view flash].
	self send: #sendersOfIt: toModelWith: {selector} orDo: [super sendersOfIt]! !

!OBTextMorphEditor methodsFor: 'actions' stamp: 'cwp 10/31/2004 01:10'!
referencesToIt

	| selector |
	self lineSelectAndEmptyCheck: [^ self].
	(selector _ self selectedSelector) == nil ifTrue: [^ view flash].
	self send: #referencesToIt: toModelWith: {selector} orDo: [super referencesToIt]! !

!OBTextMorphEditor methodsFor: 'actions' stamp: 'cwp 10/31/2004 01:52'!
browseIt
	| symbol |
	self lineSelectAndEmptyCheck: [^ self].
	(symbol _ self selectedSymbol) isNil ifTrue: [^ view flash].

	self send: #browseIt: toModelWith: {symbol} orDo: [super browseIt]! !

!OBTextMorphEditor methodsFor: 'actions' stamp: 'cwp 10/31/2004 00:40'!
implementorsOfIt
	"Open a senders browser on the selected selector"

	| selector |
	self lineSelectAndEmptyCheck: [^ self].
	(selector _ self selectedSelector) == nil ifTrue: [^ view flash].
	self send: #implementorsOfIt: toModelWith: {selector} orDo: [super sendersOfIt]! !

!OBTextMorphEditor methodsFor: 'model access' stamp: 'cwp 10/31/2004 00:24'!
send: aSelector toModelWith: args orDo: aBlock
	self terminateAndInitializeAround:
		[(model respondsTo: aSelector)
			ifTrue: [(model perform: aSelector withArguments: args)
						ifFalse: [view flash]]
			ifFalse: aBlock]! !

RectangleMorph subclass: #OBPane
	instanceVariableNames: 'model list button'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBPane commentStamp: 'cwp 3/5/2004 11:52' prior: 0!
An OBPane is the visual representation of a column in a browser. It contains a morph to display nodes (typically a PluggableListMorph) and (optionally) a morph for communicating with the column's filter. It's main responsibility is to layout it's submorphs as the filter controls are added and removed.

iVars:

model	- the OBColumn that controls the node list displayed in this pane
list		- the morph which displays the node list, usually a PluggableListMorph
button	- the morph which controls the column's filter, usually an OBRadioButtonBar.!

!OBPane methodsFor: 'updating' stamp: 'cwp 11/2/2004 00:53'!
addButton: aButton
	self hasButton ifTrue: [self removeMorph: button].
	button _ aButton.
	button height: self defaultButtonHeight.
	self 
		addMorph: button;
		adjustList;
		adjustButton! !

!OBPane methodsFor: 'updating' stamp: 'cwp 2/12/2004 18:35'!
adjustList
	self hasButton
			ifFalse: [list bounds: self innerBounds]
			ifTrue: [list bounds: (self innerBounds withHeight: (self height - self buttonHeight))]! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 19:17'!
setColumn: aColumn filter: aFilter
	self model: aColumn.
	list _ aColumn listMorph.
	self initGeometry.
	aFilter ifNotNil: [self addButton: aFilter buttonMorph].! !

!OBPane methodsFor: 'geometry' stamp: 'cwp 2/12/2004 18:32'!
bounds: aRectangle
	super bounds: aRectangle.
	self adjustList.
	self adjustButton.! !

!OBPane class methodsFor: 'instance creation' stamp: 'cwp 2/12/2004 18:12'!
forColumn: aColumn withFilter: aFilter
	^ self new setColumn: aColumn filter: aFilter! !

!OBPane methodsFor: 'constructing' stamp: 'cwp 11/27/2004 21:19'!
noteNewOwner: aMorph
	self containingWindow ifNotNilDo: [:window | self adoptPaneColor: window paneColor]! !

!OBPane methodsFor: 'updating' stamp: 'cwp 2/12/2004 20:29'!
removeButton
	self hasButton 
		ifTrue: [self removeMorph: button.
				button _ nil.
				self adjustList]! !

!OBPane class methodsFor: 'instance creation' stamp: 'cwp 2/12/2004 18:11'!
forColumn: aColumn
	^ self forColumn: aColumn withFilter: nil! !

!OBPane methodsFor: 'constructing' stamp: 'cwp 11/2/2004 01:10'!
defaultButtonHeight
	^ 23! !

!OBPane methodsFor: 'testing' stamp: 'cwp 2/12/2004 18:35'!
hasButton
	^ button notNil! !

!OBPane methodsFor: 'updating' stamp: 'cwp 2/12/2004 20:23'!
update: aSelector
	aSelector = #filter ifFalse: [^ self].
	(model wantsButton)
			ifTrue: [self addButton: model filter buttonMorph]
			ifFalse: [self removeButton]! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:26'!
initGeometry
	self	
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		clipSubmorphs: true;
		color: Color transparent;
		cellInset: 0;
		borderWidth: 0;
		layoutPolicy: ProportionalLayout new.
		
	self addMorph: list.
	list bounds: self innerBounds.
		! !

!OBPane methodsFor: 'initialization' stamp: 'cwp 2/12/2004 18:44'!
model: anObject
	"Set my model and make me me a dependent of the given object."

	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].
	model _ anObject.
! !

!OBPane methodsFor: 'updating' stamp: 'cwp 11/2/2004 00:50'!
adjustButton
	| inner |
	self hasButton ifTrue: 
		[inner _ self innerBounds.
		button bounds: (inner withTop: inner bottom - self buttonHeight)]! !

!OBPane methodsFor: 'constructing' stamp: 'cwp 2/12/2004 18:14'!
listMorphForColumn: aColumn
	^ (PluggableListMorph
		on: aColumn
		list: #list
		selected: #selection
		changeSelected: #selection:
		menu: #menu:
		keystroke: #keystroke:from:)
			borderWidth: 0! !

!OBPane methodsFor: 'constructing' stamp: 'cwp 11/2/2004 00:57'!
buttonHeight
	^ self hasButton
		ifTrue: [button height]
		ifFalse: [self defaultButtonHeight]
! !

RectangleMorph subclass: #OBPaneScroller
	instanceVariableNames: 'model sizing panes transform scrollBar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBPaneScroller commentStamp: 'cwp 3/5/2004 12:13' prior: 0!
Instances of OBPaneScroller contain the panes which represent columns in a browser. Their primary responsibilities are laying out panes to fit the space available and scrolling them horizontally when there isn't sufficient space. 

iVars:

sizing		- The number of panes which should exactly fit the available space.
		   	   During layout, the  width of the panes is determined accordingly.
transform	- A TransformMorph used for scrolling
scrollBar	- An OBHorizontalScrollBar used for scrolling!

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 11:10'!
adjustPaneHeight
	"This gets called after the scrollbar has been shown or hidden, to move the bottom
	of the panes to the right place."
	
	transform bounds: self innerBounds.
	transform submorphsDo: [:m | m bounds: (m bounds withHeight: self paneHeight)]
		! !

!OBPaneScroller methodsFor: 'stepping' stamp: 'cwp 11/24/2004 22:29'!
stepTime
	^ 250! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 11/22/2004 23:19'!
sizing
	^ sizing ifNil: [self updateSizing]! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/21/2004 13:51'!
scrollBarValue: value
	transform hasSubmorphs ifFalse: [^ self].
	transform offset: (self leftoverScrollRange * value) rounded @ 0.! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 11/17/2004 23:09'!
panes
	^ panes ifNil: [self updatePanes. panes]! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 21:04'!
leftoverScrollRange
	^ (self totalScrollRange - self innerBounds width roundTo: self scrollDeltaWidth) max: 0
! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 10:42'!
isScrollable
	^ self leftoverScrollRange > 0! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 11/21/2004 22:24'!
fullLayout
	self layoutWidgets.
	self layoutPanes.
	self hideOrShowScrollBar.
	self setScrollDeltas.
	self scrollToRight.! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 19:27'!
totalScrollRange
	| submorphBounds |
	submorphBounds := transform localSubmorphBounds ifNil: [^ 0].
	^ submorphBounds width
! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/26/2004 23:14'!
paneWidthsToFit: limit
	| padded |
	padded _ Array new: self paneCount.
	padded atAllPut: (limit / self sizing) floor.
	(1 to: limit - padded sum) do: [:i | padded at: i put: (padded at: i) + 1].
	^ padded
	
	! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 11:10'!
hideScrollBar
	self removeMorph: scrollBar.
	self adjustPaneHeight.! !

!OBPaneScroller methodsFor: 'initialization' stamp: 'cwp 2/8/2004 10:52'!
initializeTransform
	transform _ TransformMorph new.
	transform
		color: Color transparent;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill;
		disableTableLayout;
		bounds: super innerBounds.
	self addMorphBack: transform.
! !

!OBPaneScroller methodsFor: 'private' stamp: 'cwp 11/23/2004 01:31'!
basicUpdatePanes
	panes := model ifNotNil: [model panes] ifNil: [Array new].		
	self clearPanes.
	panes do: [:ea | self pushPane: ea].
! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 11/17/2004 22:03'!
model
	^model! !

!OBPaneScroller class methodsFor: 'as yet unclassified' stamp: 'cwp 11/17/2004 22:01'!
withModel: aModel
	^ self new model: aModel! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 12/8/2003 21:42'!
scrollToRight
	^ scrollBar setValue: 1.! !

!OBPaneScroller methodsFor: 'private' stamp: 'cwp 11/23/2004 01:14'!
basicUpdateSizing
	model ifNil: [sizing := 1] ifNotNil: [sizing := model sizing]! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/23/2004 01:23'!
reclaimPanes
	| reclaimed |
	reclaimed := model reclaimPanes.
	reclaimed isZero	ifFalse: 
		[self 
		 	popPanes: reclaimed;
			basicUpdateSizing;
			layoutPanes;
			hideOrShowScrollBar;
			setScrollDeltas]! !

!OBPaneScroller methodsFor: 'initialization' stamp: 'cwp 11/23/2004 01:28'!
initialize
	super initialize.
	self 
		color: Color red;
		borderWidth: 0;
		vResizing: #spaceFill;
		hResizing: #spaceFill.
	self 
		initializeTransform;
		initializeScrollbar.
	self startStepping.! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 11/17/2004 22:46'!
pushPane: aMorph
	aMorph 
		borderWidth: 0;
		hResizing: #rigid;
		vResizing: #rigid;
		layoutInset: 0.
	transform hasSubmorphs ifTrue: [transform addMorphBack: self separator].
	transform addMorphBack: aMorph.
! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 11:11'!
showScrollBar
	self scrollBarIsVisible ifTrue: [^ self].
	self resizeScrollBar.
	self addMorphFront: scrollBar.
	self adjustPaneHeight.
	! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 11:03'!
resizeScrollBar
	| inner outer |
	outer _ super innerBounds.
	inner _ outer withHeight: outer height - self scrollBarHeight - 1.
	scrollBar bounds: ((inner left @ inner bottom + 1)
						corner: outer bottomRight)! !

!OBPaneScroller methodsFor: 'initialization' stamp: 'cwp 2/8/2004 11:01'!
initializeScrollbar
	scrollBar _ OBScrollBar new model: self slotName: 'scrollBar'.
	scrollBar 
		borderWidth: 0;
		borderColor: #inset;
		height: self scrollBarHeight.
	self resizeScrollBar.
! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 10:44'!
hideOrShowScrollBar
	self isScrollable ifTrue: [self showScrollBar] ifFalse: [self hideScrollBar]! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 11/27/2004 01:03'!
separator
	^ BorderedSubpaneDividerMorph vertical 
		borderWidth: self separatorWidth / 2;
		color: model defaultBackgroundColor duller;
		borderRaised.! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 11:06'!
doLayout
	self layoutWidgets.
	self layoutPanes.
	self hideOrShowScrollBar.
	self setScrollDeltas.
	self scrollToRight.! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/25/2003 21:14'!
setScrollDeltas
	| range interval value |
	transform hasSubmorphs ifFalse: [scrollBar interval: 1.0. ^ self].
	range _ self leftoverScrollRange.
	range = 0 ifTrue: [^ scrollBar interval: 1.0; setValue: 0].
	interval _ ((self innerBounds width) / self totalScrollRange) asFloat.
	value _ (transform offset x / range min: 1.0) asFloat.
	scrollBar interval: interval.
	scrollBar setValue: value.! !

!OBPaneScroller methodsFor: 'defaults' stamp: 'cwp 8/24/2003 15:13'!
separatorWidth
	^ 4! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 8/25/2003 22:13'!
layoutWidgets
	| inner outer |
	outer _ super innerBounds.
	inner _ self innerBounds.
	transform bounds: inner.
	scrollBar bounds: ((inner left @ inner bottom + 1)
						corner: outer bottomRight)! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 11/17/2004 23:09'!
paneCount
	^ self panes size! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/23/2004 01:34'!
updatePanes
	| count |
	model ifNil: [panes := Array new. ^ self].
	count := panes ifNotNil: [panes size] ifNil: [0].
	self basicUpdatePanes.
	self basicUpdateSizing.	
	self layoutPanes.
	panes size = count
		ifFalse: [self hideOrShowScrollBar.
				self setScrollDeltas].
	panes size > count ifTrue: [self scrollToRight].
	^ panes! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 8/23/2003 16:21'!
scrollDeltaWidth
	^ 1! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 12/6/2003 17:06'!
clearPanes
	transform removeAllMorphs! !

!OBPaneScroller methodsFor: 'stepping' stamp: 'cwp 11/21/2004 15:38'!
step
	self reclaimPanes! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/25/2004 20:23'!
totalPaneWidth
	^ self innerBounds width - ((self sizing - 1) * self separatorWidth)! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/26/2004 23:14'!
computeMorphWidths
	| paneWidths widths |
	paneWidths _ self paneWidthsToFit: self totalPaneWidth.
	widths _ OrderedCollection new.
	paneWidths do: [:w | widths add: w] separatedBy: [widths add: self separatorWidth].
	^ widths asArray
! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 8/24/2003 14:29'!
scrollBarHeight
	^ 12! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 12/6/2003 17:08'!
paneHeight
	^ transform bounds height! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 10:54'!
innerBounds
	| rect |
	rect _ super innerBounds.
	^ self scrollBarIsVisible
		ifTrue: [rect withHeight: rect height - self scrollBarHeight - 1]
		ifFalse: [rect]! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 2/8/2004 01:22'!
layoutPanes
	| widths rect |
	widths _ self computeMorphWidths.
	rect _ 0@0 extent: (0 @ self paneHeight).
	transform submorphs 
					with: widths 
					do: [:m :w | 
						rect _ rect withWidth: w.
						m bounds: rect.
						rect _ rect translateBy: (w@0)]
						
! !

!OBPaneScroller methodsFor: 'scrolling' stamp: 'cwp 2/8/2004 10:56'!
scrollBarIsVisible
	^ submorphs includes: scrollBar! !

!OBPaneScroller methodsFor: 'panes' stamp: 'cwp 11/23/2004 01:22'!
popPanes: count
	count * 2 timesRepeat: [transform removeMorph: transform lastSubmorph].
	panes removeLast: count! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/17/2004 22:40'!
update: aSymbol
	aSymbol = #sizing ifTrue: [^ self updateSizing].
	aSymbol = #panes ifTrue: [^ self updatePanes].! !

!OBPaneScroller methodsFor: 'updating' stamp: 'cwp 11/23/2004 01:14'!
updateSizing
	| old |
	old := sizing.
	self basicUpdateSizing.
	sizing = old ifFalse: [self layoutPanes].
	^sizing! !

!OBPaneScroller methodsFor: 'accessing' stamp: 'cwp 11/17/2004 22:11'!
model: anObject
	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].
	model := anObject! !

!OBPaneScroller methodsFor: 'layout' stamp: 'cwp 11/21/2004 23:45'!
bounds: aRectangle
	super bounds: aRectangle.
	self layoutWidgets.
	self layoutPanes.
	self setScrollDeltas.
! !

RectangleMorph subclass: #OBRadioButtonBar
	instanceVariableNames: 'model buttons selection getListSelector getSelectionSelector setSelectionSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBRadioButtonBar commentStamp: 'cwp 3/5/2004 11:59' prior: 0!
An OBRadioButtonBar is similar to a PluggableListMorph except that it displays a row of buttons rather than a vertical list. Clicking on a button selects it.

model				- the model for this button bar
buttons				- a collection of OBButtonModels, which are derived from the model's list
selection			- the index of the currently selected button
getListSelector	 	- the message sent to the model to get the list of labels for the buttons
getSelectionSelector	- the message sent to the model to get the index of the currently selected item
setSelectionSelector	- the message sent to the model to change the current selection after a button
					   is clicked!

!OBRadioButtonBar methodsFor: 'initialize-release' stamp: 'cwp 2/25/2004 00:36'!
on: aModel list: listSelector selected: selectionGetter changeSelected: selectionSetter 
	self model: aModel.
	selection _ 0.
	getListSelector _ listSelector.
	getSelectionSelector _ selectionGetter.
	setSelectionSelector _ selectionSetter.
	self initGeometry.
	self updateList.! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:38'!
withSelectedButtonDo: aBlock
	^ self withButtonAt: selection do: aBlock! !

!OBRadioButtonBar methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 18:47'!
push: aButton
	| index |
	index _ buttons indexOf: aButton.
	model perform: setSelectionSelector with: index.! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:39'!
updateSelection
	| oldSelection |
	oldSelection _ selection.
	selection _ self getSelectionIndex.
	self withButtonAt: oldSelection do: [:button | button selectionChanged].
	self withSelectedButtonDo: [:button | button selectionChanged]! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 11/2/2004 01:09'!
updateMorphs
	self removeAllMorphs.
	buttons do: [:button | self addMorphBack: button morph].
	self bounds: (self bounds withHeight: submorphs first height)! !

!OBRadioButtonBar methodsFor: 'callbacks' stamp: 'cwp 11/27/2004 19:18'!
isEnabled: aButton
	^ true! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 11/27/2004 17:58'!
updateList
	self 
		updateButtons; 
		updateMorphs! !

!OBRadioButtonBar methodsFor: 'initialize-release' stamp: 'cwp 11/27/2004 18:02'!
initGeometry
	self
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		rubberBandCells: true;
		borderWidth: 0.
! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 19:38'!
withButtonAt: index do: aBlock
	^ (buttons at: index ifAbsent: [nil]) ifNotNilDo: aBlock! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/25/2004 00:35'!
update: aSymbol
	aSymbol = getListSelector ifTrue: [self updateList. ^ self].
	aSymbol = getSelectionSelector ifTrue: [self updateSelection]! !

!OBRadioButtonBar class methodsFor: 'as yet unclassified' stamp: 'cwp 2/22/2004 16:44'!
on: aModel list: listSelector selected: selectionSelector changeSelected: changedSelector 
	^ self new
			on: aModel
			list: listSelector
			selected: selectionSelector
			changeSelected: changedSelector! !

!OBRadioButtonBar methodsFor: 'initialize-release' stamp: 'cwp 2/22/2004 16:51'!
model: aModel
	model ifNotNil: [model removeDependent: self].
	model _ aModel.
	model addDependent: self.! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 2/24/2004 18:53'!
getSelectionIndex
	^ model perform: getSelectionSelector! !

!OBRadioButtonBar methodsFor: 'updating' stamp: 'cwp 11/27/2004 17:58'!
updateButtons
	| labels |
	labels := model perform: getListSelector.
	buttons := Array new: labels size.
	labels withIndexDo: 
			[:label :index | 
			buttons at: index put: (OBButtonModel withLabel: label inBar: self)].
	selection := self getSelectionIndex! !

!OBRadioButtonBar methodsFor: 'constructing' stamp: 'cwp 11/27/2004 20:58'!
adoptPaneColor: aColor
	self submorphs do: [:ea | ea onColor: aColor darker offColor: aColor lighter]! !

!OBRadioButtonBar methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 18:54'!
isSelected: aButton
	^ (buttons at: selection ifAbsent: [^ false]) == aButton! !

!OBRadioButtonBar methodsFor: 'accessing' stamp: 'cwp 11/27/2004 02:09'!
list
	buttons ifNil: 
			[| labels |
			labels := model perform: getListSelector.
			buttons := Array new: labels size.
			labels withIndexDo: 
					[:label :index | 
					buttons at: index put: (OBButtonModel withLabel: label inBar: self)].
			selection := self getSelectionIndex.
			self].
	^buttons collect: [:b | b label]! !

!OBRadioButtonBar methodsFor: 'constructing' stamp: 'cwp 11/27/2004 21:02'!
noteNewOwner: aMorph
	| window |
	window := aMorph containingWindow.
	window ifNotNil: [self adoptPaneColor: window paneColor]! !

TextMorphForEditView subclass: #OBTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBTextMorph methodsFor: 'as yet unclassified' stamp: 'cwp 10/30/2004 23:06'!
installEditorToReplace: priorEditor
	"Install an editor for my paragraph.  This constitutes 'hasFocus'.
	If priorEditor is not nil, then initialize the new editor from its state.
	We may want to rework this so it actually uses the prior editor."

	| stateArray |
	priorEditor ifNotNil: [stateArray _ priorEditor stateArray].
	editor _ OBTextMorphEditor new morph: self.
	editor changeParagraph: self paragraph.
	priorEditor ifNotNil: [editor stateArrayPut: stateArray].
	self selectionChanged.
	^ editor! !

!PluggableListMorph methodsFor: '*omnibrowser' stamp: 'cwp 3/14/2004 13:09'!
getListDelicately
	| lazy |
	lazy _ self listMorph.
	^ (1 to: lazy getListSize) collect: [:i | lazy item: i].
	! !

!PluggableListMorph methodsFor: '*omnibrowser' stamp: 'cwp 3/14/2004 13:47'!
getListObtrusively
	| lazy |
	lazy _ self listMorph.
	^ (1 to: lazy getListSize) collect: [:i | lazy getListItem: i].
	! !

!LazyListMorph methodsFor: '*omnibrowser' stamp: 'cwp 2/25/2004 21:17'!
rectForRow: index
	"return a rectangle containing the row at index"
	
	| top |
	top _ self top + (index - 1 * font height).
	^ (self left @ top) extent: self width @ font height
! !

Object subclass: #OBBrowser
	instanceVariableNames: 'panels'
	classVariableNames: 'Metagraphs'
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBBrowser commentStamp: 'cwp 3/4/2004 21:30' prior: 0!
OBBrowser is the core of OmniBrower. It's the root object for each browser and the model for the SystemWindows which display them. Its main responsibilies are managing columns and distributing update messages.

The browser UI is divided into two parts. At the top are the columns, which display the structure of the node graph. As nodes are selected, additional columns are added to the browser to hold the children of the selected nodes. Below the columns are panels, which allow the user to interact with the selected node.  The only panel implemented so far is OBTextPanel, but others such as annotation panes, or button bars could be created.

iVars:

panels	- A collection of objects which manage the other (non-column) parts of the browser.
columns	- A collection of OBColumns, each of which manages a single pane in the scroller.
scroller	- The OBPaneScroller which owns the column panes.
minPanes - The minimum number of panes the browser will display.

cVars:

MetaGraphs - A dictionary matching names to metagraphs!

!OBBrowser methodsFor: 'opening' stamp: 'cwp 10/17/2004 21:31'!
open
	^ OBBrowseRequest signal: self! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 18:07'!
graph: aSymbol root: aNode 
	^ self graph: aSymbol root: aNode selection: nil! !

!OBBrowser class methodsFor: 'metagraphs' stamp: 'cwp 11/7/2004 21:22'!
metagraphs
	^ Metagraphs ifNil: [self clearMetagraphs]! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 11/20/2004 21:00'!
jumpTo: aNode 
	self navigationPanel jumpTo: aNode! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 11/26/2004 22:27'!
optionalButtonPanel
	| labels panel |
	labels _ self optionalButtons.
	(Preferences optionalButtons and: [labels isEmpty not]) ifTrue: 
		[panel _ OBFixedButtonPanel new.
		labels do: [:ea | panel addButtonWithLabel: ea]].
	^ panel! !

!OBBrowser methodsFor: 'morphic' stamp: 'cwp 11/21/2004 00:31'!
morphicPanelLayout
	^ OBMorphicPanelLayout for: panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 18:12'!
metaNode: metaNode node: rootNode 
	^ self metaNode: metaNode root: rootNode selection: nil panels: self panels! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 11/20/2004 21:00'!
root
	^self navigationPanel root! !

!OBBrowser class methodsFor: 'metagraphs' stamp: 'cwp 11/7/2004 21:18'!
removeMetagraphAt: aSymbol
	self metagraphs removeKey: aSymbol! !

!OBBrowser class methodsFor: 'metagraphs' stamp: 'cwp 11/7/2004 21:17'!
metagraphMissing: aSymbol
	^ self error: 'There is no metagraph named ', aSymbol.! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 8/31/2004 13:04'!
defaultLabel
	^ self class titleForRoot: self root! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:01'!
maxPanes
	^ self paneCount! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 12/5/2004 16:25'!
navigationPanel
	^ panels 
		detect: [:ea | ea isNavigation] 
		ifNone: [self error: 'No navigation panel configured']! !

!OBBrowser class methodsFor: 'metagraphs' stamp: 'cwp 11/16/2004 21:07'!
clearMetagraphs
	^ Metagraphs _ Dictionary new.
! !

!OBBrowser class methodsFor: 'metagraphs' stamp: 'cwp 11/7/2004 21:16'!
metagraphAt: aSymbol
	^ self metagraphs at: aSymbol ifAbsent: [self metagraphMissing: aSymbol]! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 8/31/2004 11:20'!
selection: selectedNode
	^ self root: self rootNode selection: selectedNode! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:07'!
optionalButtons
	^ #()! !

!OBBrowser class methodsFor: 'metagraphs' stamp: 'cwp 11/7/2004 21:17'!
metagraphAt: aSymbol put: aMetagraph
	self metagraphs at: aSymbol put: aMetagraph! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 11/20/2004 21:00'!
jumpToRoot
	^self navigationPanel jumpToRoot! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 9/10/2004 16:53'!
root: rootNode selection: selectedNode
	^ self graph: self metaGraphName root: rootNode selection: selectedNode! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 12/5/2004 16:24'!
initialize
	panels := OrderedCollection new! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 11/20/2004 21:00'!
currentNode
	^self navigationPanel currentNode! !

!OBBrowser methodsFor: 'morphic' stamp: 'cwp 11/27/2004 22:21'!
defaultBackgroundColor
	^ Color gray veryMuchLighter! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 18:10'!
metaNode: metaNode root: rootNode selection: selectedNode panels: panels
	| browser |
	browser _ self new.
	panels do: [:ea | browser addPanel: ea].
	browser setMetaNode: metaNode node: rootNode.
	selectedNode ifNotNil: [browser jumpTo: selectedNode].
	^ browser! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 16:55'!
navigationPanel
	^ OBColumnPanel minPanes: self minPanes maxPanes: self maxPanes! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/20/2004 21:00'!
nodeSelected: aNode 
	^self navigationPanel nodeSelected: aNode! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 18:11'!
graph: graphName root: rootNode selection: selectedNode panels: panels
	^ self
		metaNode: (self metagraphAt: graphName)
		root: rootNode
		selection: selectedNode
		panels: panels! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 13:02'!
titleForRoot: aNode
	^ self title! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 15:18'!
graph: graphName root: rootNode selection: selectedNode
	^ self
		graph: graphName
		root: rootNode
		selection: selectedNode
		panels: self panels! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 2/28/2004 14:06'!
okToChange
	panels do: [:ea | ea okToChange ifFalse: [^ false]].
	^ true! !

!OBBrowser methodsFor: 'callbacks' stamp: 'cwp 11/20/2004 21:05'!
labelString
	| label |
	label := self navigationPanel labelString.
	^label 
		ifNil: [self defaultLabel]
		ifNotNil: [self defaultLabel , ': ' , label]! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 12/5/2004 17:03'!
openRoot: rootNode selection: selectedNode
	^ (self root: rootNode selection: selectedNode) open! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/20/2004 21:00'!
nodeDeleted: aNode 
	self navigationPanel nodeDeleted: aNode! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 15:06'!
panels
	^ {self navigationPanel. self optionalButtonPanel. self definitionPanel} 
		reject: [:ea | ea isNil]! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 12/5/2004 12:59'!
refresh
	panels do: [:ea | ea refresh]! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 11/7/2004 21:11'!
defaultMetagraph
	self subclassResponsibility! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:53'!
panels
	^ panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 15:26'!
root: aNode 
	^ self root: aNode selection: nil! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:00'!
minPanes
	^ self paneCount! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 8/31/2004 11:21'!
addPanel: aPanel
	panels add: aPanel.
	aPanel browser: self! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 12/5/2004 15:25'!
open
	^ (self root: self rootNode) open! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:01'!
paneCount
	^ 4! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 13:19'!
rootNode
	self subclassResponsibility! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 11/20/2004 21:00'!
currentOrRootNode
	^self navigationPanel currentOrRootNode! !

!OBBrowser methodsFor: 'morphic' stamp: 'cwp 11/21/2004 00:30'!
openInMorphic
	^ self morph openInWorld! !

!OBBrowser methodsFor: 'morphic' stamp: 'cwp 12/5/2004 14:23'!
morph
	| window |
	window := SystemWindow labelled: self defaultLabel.
	window model: self.
	panels isEmpty ifFalse: [self morphicPanelLayout addMorphsTo: window].
	^window! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:10'!
definitionPanel
	^ OBDefinitionPanel new! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 12/5/2004 16:24'!
setMetaNode: aMetaNode node: aNode 
	self navigationPanel setMetaNode: aMetaNode node: aNode! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:32'!
title
	^ 'OmniBrowser'! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 8/28/2004 00:05'!
nodeChanged
	panels do: [:ea | ea nodeChanged].
	self changed: #relabel! !

Object subclass: #OBDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:08'!
doItContext
	^ nil! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:51'!
accept: aText notifying: aController
	^ self text: aText! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 22:16'!
selection
	^ 1 to: 0! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:16'!
doItReceiver
	^ self selectedClass ifNil: [FakeClassPool new]! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:52'!
text: aText
	^ false! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 22:16'!
textSelection
	^ self selection! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:52'!
text
	^ ''! !

!OBDefinition methodsFor: 'accessing' stamp: 'cwp 3/23/2004 00:16'!
classRef
	^ nil! !

Object subclass: #OBPanel
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:58'!
browser
	^ browser! !

OBPanel subclass: #OBColumnPanel
	instanceVariableNames: 'columns minPanes maxPanes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBColumnPanel methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:10'!
isNavigation
	^ true! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/16/2004 22:21'!
columnAfter: aColumn 
	^[self columns after: aColumn] on: Error do: [^nil]! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/17/2004 22:48'!
panes
	^ columns collect: [:ea | ea morph]! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/22/2004 23:18'!
pushColumn: aColumn 
	self columns addLast: aColumn.
	self changed: #panes.
! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:46'!
columns
	^columns! !

!OBColumnPanel class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 16:56'!
new
	^ self minPanes: 1 maxPanes: 1! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 11/20/2004 22:05'!
basicOkToChange
	^ browser okToChange! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:22'!
currentNode
	^ self currentColumn ifNotNilDo: [:column | column selectedNode]! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 23:29'!
maxPanes
	^maxPanes! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/26/2004 21:55'!
defaultBackgroundColor
	^ browser defaultBackgroundColor! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/23/2004 00:55'!
reclaimPanes
	| old |
	old := columns size.
	[self okToReclaimPane] whileTrue: [columns removeLast].
	^ old - columns size! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 11/16/2004 23:34'!
jumpToRoot
	^ self columns first selection: 0! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 11/17/2004 00:08'!
refresh
	self columns do: [:ea | ea refresh].! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 11/17/2004 00:04'!
nodeSelected: aNode
	^ self jumpTo: aNode! !

!OBColumnPanel methodsFor: 'constructing' stamp: 'cwp 11/20/2004 22:22'!
morph
	^ (OBPaneScroller withModel: self)
		name: 'scroller';
		yourself! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/22/2004 22:17'!
selected: aColumn ! !

!OBColumnPanel class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 16:56'!
minPanes: min maxPanes: max
	^ self basicNew setMinPanes: min maxPanes: max! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:54'!
minPanes
	^ minPanes! !

!OBColumnPanel methodsFor: 'constructing' stamp: 'cwp 11/20/2004 22:26'!
morphHeight
	^ 0! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/23/2004 00:52'!
okToReclaimPane
	columns size > minPanes ifFalse: [^ false].
	^ columns last isEmpty or: [(columns at: columns size - 1) shouldBeLast].
! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/18/2004 00:00'!
sizing
	^ (columns size max: minPanes) min: maxPanes! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 11/16/2004 23:33'!
jumpTo: aNode 
	| column ancestor |
	column := self columns first.
	
	[ancestor := column selectAncestorOf: aNode.
	ancestor = aNode or: [ancestor isNil]] 
			whileFalse: [column := self columns after: column]! !

!OBColumnPanel methodsFor: 'initializing' stamp: 'cwp 12/5/2004 16:50'!
setMetaNode: aMetaNode node: aNode 
	aNode metaNode: aMetaNode.
	columns add: (aMetaNode columnInPanel: self node: aNode).
	minPanes - self columns size 
		timesRepeat: [self pushColumn: self emptyColumn]! !

!OBColumnPanel methodsFor: 'initializing' stamp: 'cwp 12/5/2004 16:54'!
setMinPanes: min maxPanes: max
	columns _ OrderedCollection new.
	minPanes _ min.
	maxPanes _ max! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/16/2004 22:22'!
currentColumn
	^self columns reversed detect: [:ea | ea hasSelection] ifNone: []! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 11/20/2004 22:03'!
basicNodeChanged
	browser nodeChanged! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 11/16/2004 23:35'!
root
	^self columns first parent! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 11/20/2004 22:08'!
nodeDeleted: aNode
	aNode = self root 
		ifTrue: 
			[self columns first clear.
			self basicNodeChanged]! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:23'!
currentOrRootNode
	^(self columns reversed detect: [:ea | ea hasSelection]
		ifNone: [^self columns first parent]) selectedNode! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/16/2004 23:01'!
emptyColumn
	^ OBColumn inPanel: self! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/22/2004 23:18'!
popColumn
	self columns removeLast.
	self changed: #panes! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:46'!
columns: anObject
	columns := anObject! !

!OBColumnPanel methodsFor: 'constructing' stamp: 'cwp 11/17/2004 23:24'!
buildScroller
	^ (OBPaneScroller withModel: self)
		name: 'scroller';
		yourself! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 23:51'!
labelString
	| label |
	self columns reversed do: 
		[:ea | 
		label := ea selectedNode ifNotNilDo: [:node | node title].
		label ifNotNil: [^ label]].
	^ nil
! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/22/2004 22:17'!
deselected: aColumn 
! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 3/9/2004 23:52'!
nodeChanged
	! !

!OBPanel methodsFor: 'accessing' stamp: 'cwp 8/31/2004 10:45'!
browser: aBrowser
	browser _ aBrowser! !

OBPanel subclass: #OBVarButtonPanel
	instanceVariableNames: 'morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBVarButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/26/2004 22:41'!
morphHeight
	^ 25! !

!OBVarButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/27/2004 18:16'!
morph
	^ morph _ (RectangleMorph new)
					layoutPolicy: TableLayout new;
					listDirection: #leftToRight;
					hResizing: #spaceFill;
					vResizing: #spaceFill;
					borderWidth: 0;
					wrapCentering: #center;
					cellPositioning: #leftCenter;
					rubberBandCells: true;
					yourself! !

!OBVarButtonPanel methodsFor: 'updating' stamp: 'cwp 11/27/2004 18:23'!
nodeChanged
	| node actions |
	morph removeAllMorphs.
	node _ browser currentNode ifNil: [^ self].
	actions _ (node metaNode actionsForNode: node) select: [:ea | ea wantsButton].
	actions do: [:ea | morph addMorphBack: ea button]! !

!OBPanel class methodsFor: 'instance creation' stamp: 'cwp 8/31/2004 10:46'!
inBrowser: aBrowser
	^ self new browser: aBrowser! !

!OBPanel methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:09'!
isNavigation
	^ false! !

OBPanel subclass: #OBDefinitionPanel
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBDefinitionPanel commentStamp: 'cwp 3/5/2004 14:34' prior: 0!
OBTextPanel serves as the model for the text pane of the browser. It's main responsibility is to act as a relay between the currently selected node and a PluggableTextMorph.

iVars:

browser - The browser of which this panel is a part.
!

!OBDefinitionPanel methodsFor: 'testing' stamp: 'cwp 9/16/2004 15:22'!
isDefinitionPanel
	^ true! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 3/14/2004 15:02'!
isDirty
	^ self canDiscardEdits 
	! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:46'!
accept: aText notifying: aController
	^ self 
		withDefinitionDo: [:def | [def accept: aText notifying: aController]
									on: OBNotification
									do: [:notification | notification notify: browser; resume]]
		ifNil: [true]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 21:15'!
selection
	^ self withDefinitionDo: [:def | def textSelection] ifNil: [1 to: 0]! !

!OBDefinitionPanel methodsFor: 'constructing' stamp: 'cwp 10/30/2004 22:03'!
morph
	^ (OBPluggableTextMorph
		on: self
		text: #text
		accept: #accept:notifying:
		readSelection: #selection
		menu: #menu:shifted:)
			font: Preferences standardCodeFont;
			yourself
			
	"see CodeHolder>>buildMorphicCodePaneWith:"! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:42'!
text
	^ self withDefinitionDo: [:def | def text] ifNil: ['']! !

!OBDefinitionPanel methodsFor: 'constructing' stamp: 'cwp 10/24/2004 23:28'!
yellowButtonMenu
	^ {
		{'find...(f)' translated.				#find}.			     
		{'find again (g)' translated.			#findAgain}.		     
		{'set search string (h)' translated.	#setSearchString}.
		#-.	     
		{'do again (j)' translated.			#again}.		     
		{'undo (z)' translated.				#undo}.			     
		#-.	     
		{'copy (c)' translated.				#copySelection}.	     
		{'cut (x)' translated.					#cut}.			     
		{'paste (v)' translated.				#paste}.		     
		{'paste...' translated.					#pasteRecent}.		     
		#-.	     
		{'accept (s)' translated.				#accept}.		     
		{'cancel (l)' translated.				#cancel}.		     
		#-.	     
		{'do it (d)' translated.				#doIt}.			     
		{'print it (p)' translated.				#printIt}.		     
		{'inspect it (i)' translated.			#inspectIt}.		     
		{'explore it (I)' translated.			#exploreIt}.		     
		{'debug it' translated.				#debugIt}.		     
		#-.	     
		{'browse it (b)' translated.					#browseIt}.
		{'senders of it (n)' translated.				#sendersOfIt}.
		{'implementors of it (m)' translated.			#implementorsOfIt}.
		{'references to it (N)' translated.				#referencesToIt}.
		#-.
		{'more...' translated.					#shiftedTextPaneMenuRequest}.
	}
! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 3/22/2004 20:41'!
withDefinitionDo: workBlock ifNil: nilBlock
	definition ifNil: [ ^ nilBlock value].
	^ workBlock value: definition! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:23'!
doItContext
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #doItContext) ifTrue: [def doItContext]]
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 10/24/2004 09:16'!
menu: aMenu shifted: aBoolean
	| items |
	items _  aBoolean 
		ifTrue: [self shiftedYellowButtonMenu] 
		ifFalse: [self yellowButtonMenu].
	items do: [:ea |
			  ea = #-
				ifFalse: [aMenu add: ea first action: ea second]
				ifTrue: [aMenu addLine]].
	^ aMenu ! !

!OBDefinitionPanel methodsFor: 'constructing' stamp: 'cwp 3/9/2004 21:35'!
morphHeight
	^ 0! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:56'!
definition: aDefinition
	definition _ aDefinition.
	self changed: #text! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:23'!
doItReceiver
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #doItReceiver) ifTrue: [def doItReceiver]]
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 9/17/2004 00:56'!
nodeChanged
	self definition: self getDefinition! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 12/5/2004 15:32'!
refresh
	| oldDefinition |
	definition ifNil: [^ self].
	oldDefinition _ definition.
	definition _ self getDefinition.
	definition text = oldDefinition text ifTrue: [^ self].
	self canDiscardEdits
			ifTrue: [self changed: #text]
			ifFalse: [self changed: #codeChangedElsewhere]! !

!OBDefinitionPanel methodsFor: 'constructing' stamp: 'cwp 10/24/2004 23:28'!
shiftedYellowButtonMenu
	^ {
		{'explain' translated.						#explain}.
		{'recognizer (r)' translated.					#recognizeCharacters}.
		{'verify spelling of word' translated.		#verifyWordSpelling}.
		#-.
		{'selectors containing it (W)' translated.		#methodNamesContainingIt}.
		{'method strings with it (E)' translated.		#methodStringsContainingit}.
		{'method source with it' translated.			#methodSourceContainingIt}.
		{'class names containing it' translated.		#classNamesContainingIt}.
		{'class comments with it' translated.			#classCommentsContainingIt}.
		{'change sets with it' translated.				#browseChangeSetsWithSelector}.
		#-.
		{'save contents to file...' translated.			#saveContentsInFile}.
		#-.
		{'more...' translated.							#yellowButtonActivity}.
	}
! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 10/24/2004 09:27'!
perform: aSelector orSendTo: anObject
	| receiver |
	receiver  _ (self respondsTo: aSelector) ifTrue: [self] ifFalse: [anObject].
	receiver perform: aSelector! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 3/22/2004 20:06'!
getDefinition
	^ browser currentNode ifNotNilDo: [:node | node definition]! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 2/28/2004 14:18'!
okToChange
	self canDiscardEdits ifTrue: [^ true].
	ok _ OBConfirmationRequest
			prompt: 'Changes have not been saved.
Is it OK to discard those changes?'
			confirm: 'Discard changes'.
	ok ifTrue: [self changed: #clearUserEdits].
	^  ok.! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:22'!
selectedClass
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #selectedClass) ifTrue: [def selectedClass]] 
		ifNil: [nil]! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 12/5/2004 13:07'!
refresh! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 3/14/2004 15:02'!
isDirty
	^ false! !

!OBPanel methodsFor: 'testing' stamp: 'cwp 9/16/2004 15:21'!
isDefinitionPanel
	^ false! !

OBPanel subclass: #OBFixedButtonPanel
	instanceVariableNames: 'models actions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBFixedButtonPanel methodsFor: 'accessing' stamp: 'cwp 11/26/2004 22:53'!
buttonModels
	^ models! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 8/29/2004 11:28'!
push: aModel
	(actions at: aModel) ifNotNilDo: [:action | action trigger]! !

!OBFixedButtonPanel methodsFor: 'initializing' stamp: 'cwp 11/26/2004 22:53'!
initialize
	models _ OrderedCollection new.
	actions _ Dictionary new.! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/26/2004 22:40'!
createMorph
	^(RectangleMorph new)
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 0;
		wrapCentering: #center;
		cellPositioning: #leftCenter;
		rubberBandCells: true;
		yourself! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/27/2004 21:24'!
morph
	| morph |
	morph := self createMorph.
	self buttonModels do: [:ea | morph addMorphBack: (self morphForModel: ea)].
	^morph! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/27/2004 21:25'!
morphForModel: aModel
	| morph paneColor |
	morph _ aModel morph.
	paneColor _ browser defaultBackgroundColor.
	morph onColor: paneColor darker offColor: paneColor lighter.
	^ morph! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 8/29/2004 13:47'!
nodeChanged
	| node nodeActions |
	node _ browser currentNode ifNil: [^ self].
	nodeActions _ (node metaNode actionsForNode: node) select: [:ea | ea wantsButton].
	actions keys do: 
		[:model | 
		actions at: model put: nil.
		nodeActions do: 
			[:action | model label = action buttonLabel ifTrue: [actions at: model put: action]]].
	actions keys do: [:ea | ea selectionChanged].! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 11/26/2004 22:41'!
morphHeight
	^ 25! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 11/27/2004 19:20'!
isSelected: aModel
	^ true! !

!OBFixedButtonPanel methodsFor: 'public' stamp: 'cwp 11/26/2004 22:52'!
addButtonWithLabel: label
	| model |
	model _ OBButtonModel withLabel: label inBar: self.
	models add: model.
	actions at: model put: nil! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 11/27/2004 19:20'!
isEnabled: aModel
	^ (actions at: aModel) notNil! !

Object subclass: #OBMorphicPanelLayout
	instanceVariableNames: 'panels'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBMorphicPanelLayout methodsFor: 'private' stamp: 'cwp 11/21/2004 00:25'!
addNavMorphTo: aMorph
	aMorph 
		addMorph: self navigationPanel buildScroller
		fullFrame: (LayoutFrame fractions: (0 @ 0 extent: 1 @ self columnProportion)).
! !

!OBMorphicPanelLayout class methodsFor: 'intance creation' stamp: 'cwp 11/20/2004 23:32'!
for: aCollection
	^ self new setPanels: aCollection! !

!OBMorphicPanelLayout methodsFor: 'private' stamp: 'cwp 11/21/2004 00:37'!
addOtherMorphsTo: aMorph
	| heights variableCount fraction offset vExtent h nonNavPanels |
	nonNavPanels := panels reject: [:ea | ea isNavigation].
	nonNavPanels isEmpty ifTrue: [^ self].
	heights _ nonNavPanels collect: [:ea | ea morphHeight].
	variableCount _ heights 
						inject: 0 
						into: [:count :ea | ea = 0 ifTrue: [count + 1] ifFalse: [count]].
	fraction _ self columnProportion.
	offset _ 0.
	vExtent _  1- fraction / variableCount.
	nonNavPanels with: heights do:  [:panel :height | 
		h _ height = 0 ifTrue: [vExtent] ifFalse: [0].
		aMorph 
			addMorph: panel morph
			fullFrame: (LayoutFrame
							fractions: (0@fraction extent: 1@h)
							offsets: (0@offset corner: 0@height)).
		height = 0 
			ifTrue: 	[offset _ 0.
					fraction _ fraction + vExtent]
			ifFalse: 	[offset _ offset + height]]
						
		
						! !

!OBMorphicPanelLayout methodsFor: 'private' stamp: 'cwp 11/21/2004 00:28'!
navigationPanel
	^ panels detect: [:ea | ea isNavigation]! !

!OBMorphicPanelLayout methodsFor: 'private' stamp: 'cwp 11/21/2004 00:26'!
columnProportion
	^ panels size = 1
		ifTrue: [1]
		ifFalse: [0.4]! !

!OBMorphicPanelLayout methodsFor: 'initializing' stamp: 'cwp 11/21/2004 00:23'!
setPanels: aCollection 
	panels := aCollection! !

!OBMorphicPanelLayout methodsFor: 'public' stamp: 'cwp 11/21/2004 00:26'!
addMorphsTo: aMorph
	self addNavMorphTo: aMorph.
	(panels size > 1) ifTrue: [self addOtherMorphsTo: aMorph].
	! !

Object subclass: #OBButtonModel
	instanceVariableNames: 'bar label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBButtonModel commentStamp: 'cwp 3/5/2004 12:15' prior: 0!
An OBButtonModel serves as a model for PluggableButtonMorphs used OBRadioButtonBar. OBRadioButtonBar cannot be a direct model for its PBMs, since it can contain a variable number of buttons.

iVars:

bar		- the OBRadioButton bar to which this button belongs
label	- the label of the button!

!OBButtonModel methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 18:42'!
isSelected
	^ bar isSelected: self! !

!OBButtonModel methodsFor: 'morphs' stamp: 'cwp 11/27/2004 19:06'!
morph
	| morph |
	morph _ PluggableButtonMorph
				on: self
				getState: #isSelected
				action: #push
				label: #labelMorph.
	morph 
		borderWidth: 2;
		borderRaised;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	^ morph! !

!OBButtonModel class methodsFor: 'as yet unclassified' stamp: 'cwp 8/29/2004 13:30'!
onColor
	^ Color lightGray lighter! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 2/24/2004 18:29'!
bar: aRadioButtonBar
	bar _ aRadioButtonBar! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 11/27/2004 00:50'!
selectionChanged
	self changed: #isSelected.
	self changed: #labelMorph! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 3/2/2004 21:46'!
label
	^ label! !

!OBButtonModel methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 19:30'!
push
	bar push: self.! !

!OBButtonModel methodsFor: 'callbacks' stamp: 'avi 11/29/2004 21:56'!
labelMorph
	^ (StringMorph 
		contents: self label 
		font: Preferences standardDefaultTextFont)
			color: (self isEnabled ifTrue: [Color black] ifFalse: [Color gray]);
			yourself! !

!OBButtonModel methodsFor: 'accessing' stamp: 'cwp 2/24/2004 18:29'!
label: aString
	label _ aString! !

!OBButtonModel methodsFor: 'testing' stamp: 'cwp 11/27/2004 19:09'!
isEnabled
	^ bar isEnabled: self! !

!OBButtonModel class methodsFor: 'as yet unclassified' stamp: 'cwp 2/24/2004 18:35'!
withLabel: aString inBar: aRadioButtonBar
	^ self new label: aString; bar: aRadioButtonBar! !

!OBButtonModel class methodsFor: 'as yet unclassified' stamp: 'cwp 8/29/2004 13:31'!
offColor
	^ Color lightGray twiceLighter! !

Object subclass: #OBNode
	instanceVariableNames: 'metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBNode commentStamp: 'cwp 3/4/2004 22:20' prior: 0!
A node is a wrapper for an object in the browser's domain graph. OBNode is an abstract superclass for concrete nodes which might appear in the browser.

iVars:

metaNode - the MetaNode which produced this node!

!OBNode methodsFor: 'public' stamp: 'cwp 1/31/2004 14:57'!
hash
	^ self name hash! !

!OBNode methodsFor: 'public' stamp: 'cwp 1/31/2004 14:57'!
= other
	^ self class = other class and: [self name = other name]! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/10/2003 22:27'!
name
	self subclassResponsibility! !

!OBNode methodsFor: 'notification' stamp: 'cwp 3/14/2004 16:57'!
signalDeletion
	(OBNodeDeleted for: self) signal! !

!OBNode methodsFor: 'public' stamp: 'cwp 3/22/2004 20:48'!
definition
	^ self! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBNode methodsFor: 'utility' stamp: 'cwp 2/3/2004 22:29'!
action: aSelector withLabel: aString
	^ OBAction
		label: aString
		receiver: self
		selector: aSelector
		arguments: #()! !

!OBNode methodsFor: 'displaying' stamp: 'cwp 8/28/2004 00:14'!
title
	^ nil! !

!OBNode methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:00'!
asDraggableMorph
	^(StringMorph contents: self name)
		color: Color white;
		yourself! !

OBNode subclass: #OBCollectionNode
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBCollectionNode class methodsFor: 'as yet unclassified' stamp: 'cwp 3/15/2004 23:17'!
on: aCollection
	^ self new setCollection: aCollection! !

!OBCollectionNode methodsFor: 'initalizing' stamp: 'cwp 3/15/2004 23:18'!
setCollection: aCollection
	collection _ aCollection! !

!OBCollectionNode methodsFor: 'displaying' stamp: 'cwp 3/15/2004 23:19'!
name
	^ collection species name! !

!OBCollectionNode methodsFor: 'navigating' stamp: 'cwp 3/15/2004 23:19'!
children
	^ collection! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
isDescendantOf: aNode
	^ false! !

OBNode subclass: #OBFakeNode
	instanceVariableNames: 'item parent children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFakeNode methodsFor: 'public' stamp: 'cwp 1/23/2004 21:42'!
children
	^ children 
		ifNil: [children _ (self item endsWith: 'c')
					ifTrue: [Array new]
					ifFalse: [#($a $b $c) collect: [:i | OBFakeNode parent: self item: i asString]]]! !

!OBFakeNode class methodsFor: 'instance creation' stamp: 'cwp 12/15/2003 22:06'!
parent: aFakeNode item: aString
	^ self new parent: aFakeNode; item: aString! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 16:58'!
deleteRoot
	| node |
	node _ self parent.
	[node parent notNil] whileTrue: [node _ node parent].
	node signalDeletion.! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 17:12'!
delete
	parent ifNotNil: [parent removeChild: self].
	self signalDeletion! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 2/25/2004 21:53'!
removeChild: aFakeNode
	children _ self children copyWithout: aFakeNode! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 3/14/2004 13:56'!
text: aText
	self item:( aText asString allButFirst: (self parentName size)).
	self signalSelection.
	^ true! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 16:49'!
actions
	^ Array 
		with: (self action: #beZ withLabel: 'become z')
		with: (self action: #delete withLabel: 'delete')
		with: (self action: #deleteRoot withLabel: 'delete root')! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/15/2003 22:01'!
parent: aFakeNode
	parent _ aFakeNode! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 2/25/2004 22:39'!
adopt: aNode
	aNode parent removeChild: aNode.
	aNode parent: self.
	children _ self children copyWith: aNode.
	self signalSelection! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/7/2003 19:09'!
item: aSymbol
	item _ aSymbol! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 12/15/2003 22:05'!
name
	^ self parentName, self item ! !

!OBFakeNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:40'!
dropSelector
	^ #dropOnFakeNode:! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 2/25/2004 21:49'!
parent
	^ parent! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 12/5/2004 16:16'!
isAncestorOf: aNode
	| ancestor |
	aNode name = self name ifTrue: [^ true].
	ancestor _ aNode.
	[ancestor isNil]
		whileFalse: [ancestor == self ifTrue: [^ true].
					ancestor _ ancestor parent].
	^ false! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/15/2003 22:04'!
parentName
	^ (parent ifNil: [''] ifNotNil: [parent name])! !

!OBFakeNode class methodsFor: 'browsing' stamp: 'cwp 11/7/2004 21:26'!
metagraph
	| fake |
	fake _ OBMetaNode new.
	fake 
		childAt: #children put: fake;
		addActor: (OBNodeActor onNodeClass: OBFakeNode).
	^ fake! !

!OBFakeNode class methodsFor: 'browsing' stamp: 'cwp 12/5/2004 18:37'!
browse
	| browser |
	browser _ (OBBrowser metaNode: self defaultMetagraph node: OBFakeNode new).
	browser open! !

!OBFakeNode methodsFor: 'private' stamp: 'cwp 12/7/2003 19:18'!
item
	^ item ifNil: [item _ '']! !

!OBFakeNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:41'!
dropOnFakeNode: aNode
	aNode adopt: self! !

!OBFakeNode methodsFor: 'public' stamp: 'cwp 12/15/2003 22:03'!
text
	^ self name! !

!OBFakeNode methodsFor: 'actions' stamp: 'cwp 3/14/2004 13:50'!
beZ
	item _ 'z'.
	OBListChanged signal! !

!OBNode methodsFor: 'utility' stamp: 'hpt 5/17/2004 14:28'!
action: aSelector withLabel: aString withKeystroke: aChar
	^ OBAction
		label: aString
		receiver: self
		selector: aSelector
		arguments: #()
		keystroke: aChar! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 13:49'!
accept: aText notifying: aController
	^ self text: aText! !

!OBNode methodsFor: 'notification' stamp: 'cwp 2/25/2004 22:37'!
signalSelection
	^ (OBNodeSelected for: self) signal! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/7/2003 19:27'!
text
	^ ''! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
ancestrySelector
	^ #isDescendantOf: ! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 18:30'!
textSelection
	^ 1 to: 0! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:28'!
acceptDroppedNode: aNode
 	^ aNode perform: self dropSelector with: self
! !

!OBNode methodsFor: 'public' stamp: 'cwp 1/7/2004 09:25'!
actions
	^ #()! !

!OBNode methodsFor: 'public' stamp: 'cwp 3/14/2004 14:09'!
text: aText
	^ false! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 22:11'!
fromAssociation: anAssociationNode
	^ self on: anAssociationNode value! !

!OBNode methodsFor: 'utility' stamp: 'cwp 3/10/2004 00:29'!
action: aSelector buttonLabel: buttonString menuLabel: menuString
	^ OBAction
		label: menuString
		buttonLabel: buttonString
		receiver: self
		selector: aSelector
		arguments: #()! !

!OBNode methodsFor: 'utility' stamp: 'cwp 2/7/2004 21:29'!
referenceForMethod: selector ofClass: className
	| classIsMeta symbol |
	classIsMeta _ className endsWith: ' class'.
	symbol _ classIsMeta ifTrue: [(className allButLast: 6) asSymbol] ifFalse: [className].
	^ MethodReference new 
		setClassSymbol: symbol
		classIsMeta: classIsMeta
		methodSymbol: selector
		stringVersion: symbol, '>>', selector! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode
	^ metaNode! !

!OBNode methodsFor: 'utility' stamp: 'hpt 5/17/2004 16:51'!
action: aSelector withLabel: aString withKeystroke: aChar withIcon: anIcon
	^ OBAction
		label: aString
		receiver: self
		selector: aSelector
		arguments: #()
		keystroke: aChar
		icon: anIcon! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:47'!
isAncestorOf: aNode
	^ aNode perform: self ancestrySelector with: self! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:28'!
wantsDroppedNode: aNode
	^ aNode respondsTo: self dropSelector! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'cwp 2/5/2004 20:25'!
actionsForParent: aNode
	^ #()! !

!OBNode methodsFor: 'utility' stamp: 'hpt 5/17/2004 16:51'!
action: aSelector withLabel: aString withIcon: anIcon
	^ OBAction
		label: aString
		receiver: self
		selector: aSelector
		arguments: #()
		icon: anIcon! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:29'!
dropSelector
	"Override in subclasses"
	
	^ #dropOnNode: ! !

Object subclass: #OBProtocolMock
	instanceVariableNames: 'messages answers exceptions recorder isRecording'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/13/2004 00:41'!
resume: aNotificationClass with: aValue
	exceptions ifNil: [exceptions _ OrderedCollection new].
	exceptions add: (Array with: aNotificationClass with: aValue)! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:30'!
deny: aBoolean
	self assert: aBoolean not! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/29/2004 15:49'!
handleException: exception
	| expectation |
	(exception isKindOf: TestFailure) ifTrue: [exception pass].
	(exceptions isNil or: [exceptions isEmpty]) ifTrue: [exception pass].
	expectation _ exceptions removeFirst.
	exception class = expectation first ifFalse: [exception pass].
	exception isResumable ifTrue: [expectation second
									ifNil: [exception resume]
									ifNotNil: [exception resume: expectation second]]! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:08'!
fail: aString
	^ TestResult failure sunitSignalWith: aString! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 3/14/2004 15:35'!
do: aBlock
	^ aBlock value: self recorder! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/28/2004 22:04'!
resume: aNotificationClass 
	^ self resume: aNotificationClass with: nil! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/13/2004 01:41'!
examine: aBlock
	[self do: aBlock]
		on: Exception
		do: [:exception | self handleException: exception].

	self assert: messages isEmpty
		message: 'Expected message not sent'.
	self deny: (exceptions notNil and: [exceptions notEmpty])
		message: 'Expected exception not raised'! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:29'!
deny: aBoolean message: aString
	self assert: aBoolean not message: aString! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 3/14/2004 15:36'!
recorder
	^ recorder ifNil: [recorder _ (OBMessageRecorder forMock: self)]! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:29'!
assert: aBoolean
	self assert: aBoolean message: 'Assertion failed'! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 3/14/2004 15:39'!
answer
	^ answers 
		ifNil: [self]
		ifNotNil: [answers isEmpty 
							ifTrue: [self recorder]
							ifFalse: [answers removeFirst]]! !

!OBProtocolMock methodsFor: 'recording' stamp: 'cwp 2/28/2004 17:26'!
message: aMessage
	| expected |
	isRecording ifTrue: [messages add: aMessage. ^ self].
	
	self 
		deny: messages isEmpty 
		message: 'No more messages expected'.
	expected _ messages removeFirst.
	self assert: (expected sends: aMessage selector)
		message: 'Unexpected message'.
	self assert: expected arguments = aMessage arguments
		message: 'Incorrect arguments'.
		
	^ self answer! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/28/2004 17:22'!
answer: anObjectOrCollection
	answers ifNil: [answers _ OrderedCollection new].
	anObjectOrCollection isCollection
		ifTrue: [answers addAll: anObjectOrCollection]
		ifFalse: [answers add: anObjectOrCollection]! !

!OBProtocolMock methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:28'!
assert: aBoolean message: aString
	aBoolean ifFalse: [self fail: aString]! !

!OBProtocolMock methodsFor: 'examining' stamp: 'cwp 2/13/2004 00:51'!
expect: aBlock
	isRecording _ true.
	messages _ OrderedCollection new.
	self do: aBlock.
	isRecording _ false.! !

Object subclass: #OBMetaNode
	instanceVariableNames: 'name filterClass columnClass edges actors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaNode commentStamp: 'cwp 3/5/2004 11:19' prior: 0!
A MetaNode represents a hypothetical node in the browser's domain graph.

iVars:

filterClass	- the class used to filter this hypothetical node's children
columnClass	- the class of column used to display this node's children
edges		- messages that could be sent to the node to obtain children
actors		- providers of functionality for manipulating the node!

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:44'!
children
	^ edges collect: [:edge | edge metaNode]! !

!OBMetaNode methodsFor: 'columns' stamp: 'cwp 11/16/2004 22:03'!
columnInPanel: aBrowser node: aNode
	columnClass ifNil: [columnClass _ OBColumn].
	^ columnClass
		inPanel: aBrowser
		metaNode: self
		node: aNode! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 9/14/2004 18:59'!
actionSetsForNode: aNode
	^ actors collect: [:actor | actor actionsForNode: aNode]! !

!OBMetaNode methodsFor: 'initializing' stamp: 'cwp 3/3/2004 23:04'!
initialize
	edges _ OrderedCollection new.
	actors _ OrderedCollection new.! !

!OBMetaNode methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:22'!
nodesForParent: aNode
	^ edges gather: [:edge | edge nodesForParent: aNode]! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:26'!
filter
	filterClass ifNil: [filterClass _ OBFilter].
	^ filterClass forMetaNode: self! !

!OBMetaNode methodsFor: 'columns' stamp: 'avi 3/4/2004 03:11'!
columnClass: aClass
	columnClass _ aClass! !

!OBMetaNode class methodsFor: 'instance creation' stamp: 'cwp 3/6/2004 21:06'!
named: aString
	^ self new name: aString! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:35'!
childAt: aSelector put: aMetaNode
	edges add: (OBMetaEdge selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'converting' stamp: 'cwp 3/2/2004 22:17'!
asNode
	^ OBMetaNodeNode on: self! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 9/14/2004 19:02'!
actionsForParent: aNode
	^ (self actionSetsForParent: aNode) gather: [:ea | ea]! !

!OBMetaNode methodsFor: 'metagraph' stamp: 'cwp 2/9/2004 21:12'!
edges
	^ edges! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 9/14/2004 19:00'!
actionsForNode: aNode
	^ (self actionSetsForNode: aNode) gather: [:ea | ea]! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 3/3/2004 22:48'!
addActor: anActor
	actors add: anActor.! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:02'!
filterClass: aClass
	filterClass _ aClass! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:38'!
hasChildren
	^ edges isEmpty not! !

!OBMetaNode class methodsFor: 'instance creation' stamp: 'cwp 3/3/2004 23:08'!
new
	^ self basicNew initialize! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:00'!
metaNode
	^ self! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:05'!
name: aString
	name _ aString! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/9/2004 20:28'!
childAt: aSelector labeled: aString put: aMetaNode
	edges add: (OBMetaEdge label: aString selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'actors' stamp: 'cwp 9/14/2004 19:01'!
actionSetsForParent: aNode
	^ actors collect: [:actor | actor actionsForParent: aNode]! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:54'!
name
	^ name ifNil: ['MetaNode']! !

Object subclass: #OBMetaEdge
	instanceVariableNames: 'label selector metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaEdge commentStamp: 'cwp 3/4/2004 22:02' prior: 0!
An OBMetaEdge represents an edge in the browser's metagraph. It represents a message sent to a node to obtain further nodes. It is refered to by the "parent" metanode, and refers to the "child" metanode.

iVars:

label 		-  a string describing the metaNode, for filters which allow the user 
			   to choose which edges to follow
selector		- when a node is selected by the user, this message will be sent to it to obtain its children
metaNode 	- a MetaNode corresponding to the nodes answered by the above message
		!

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 3/3/2004 00:12'!
nodesForParent: aNode
	^ (aNode perform: selector) do: [:ea | ea metaNode: metaNode]
! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
label
	^ label! !

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 22:35'!
label: aString selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aString selector: aSelector metaNode: aMetaNode! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
metaNode
	^ metaNode! !

!OBMetaEdge methodsFor: 'initializing' stamp: 'cwp 2/7/2004 22:36'!
setLabel: aString selector: aSelector metaNode: aMetaNode
	label _ aString.
	selector _ aSelector.
	metaNode _ aMetaNode! !

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2004 22:32'!
selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aSelector asString selector: aSelector metaNode: aMetaNode! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
selector
	^ selector! !

Object subclass: #OBFilter
	instanceVariableNames: 'metaNode monitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBFilter commentStamp: 'cwp 3/4/2004 21:53' prior: 0!
A browser's metagraph defines the way in which the user may traverse the graph of objects which make up the browser's domain. But it's not always desirable to have all the nodes made available by the metagraph to be visible in the browser. An filter provides a strategy for filtering out some of the nodes from the display. 

OBFilter provides a "null" filter - one that does no filtering at all - and serves as a superclass for other filters. !

!OBFilter class methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2004 20:54'!
forMetaNode: aMetaNode
	^ self new setMetaNode: aMetaNode! !

OBFilter subclass: #OBModalFilter
	instanceVariableNames: 'selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBModalFilter commentStamp: 'cwp 3/5/2004 15:16' prior: 0!
OBModalFilter is used to implement the 'instance/?/class' buttons in a standard class browser. In functional terms it filters the nodes of a column according to the edge of the metagraph that they correspond to. OBModalFilter displays an OBRadioButtonBar in its column's panel, with one button per edge. Only nodes from the currently selected edge are allowed in the column.

iVars:

selection	- the currently selected edge!

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/9/2004 21:10'!
selection
	^ selection ifNil: [selection _ 1]! !

!OBModalFilter methodsFor: 'morphs' stamp: 'cwp 11/27/2004 18:01'!
buttonMorph
	^ OBRadioButtonBar
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/25/2004 00:28'!
list
	^ metaNode edges collect: [:edge | edge label]! !

!OBModalFilter methodsFor: 'public' stamp: 'cwp 3/3/2004 00:23'!
nodesForParent: aNode
	^ self selectedEdge nodesForParent: aNode! !

!OBModalFilter methodsFor: 'public' stamp: 'cwp 2/29/2004 14:35'!
selectAncestorOf: aNode withParent: parentNode
	| ancestor |
	1 	to: metaNode edges size 
	  	do: [:i | selection _ i.
			ancestor _ (self nodesForParent: parentNode)
				detect: [:child | child isAncestorOf: aNode] 
				ifNone: [nil].
			ancestor ifNotNil: [self changed: #selection. ^ ancestor]].
	^ nil! !

!OBModalFilter methodsFor: 'selection' stamp: 'cwp 2/9/2004 21:11'!
selectedEdge
	^ metaNode edges at: self selection ! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/25/2004 00:33'!
selection: anInteger
	selection _ anInteger.
	self changed: #selection.
	monitor listChanged.
! !

!OBModalFilter methodsFor: 'selection' stamp: 'cwp 2/9/2004 21:11'!
incrementSelection
	selection _ selection + 1.
	(selection > metaNode edges size) ifTrue: [selection _ 1].! !

!OBModalFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:08'!
wantsButton
	^ true! !

!OBFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:06'!
wantsButton
	^ false! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/9/2004 20:52'!
nodesForParent: aNode
	^ metaNode nodesForParent: aNode! !

!OBFilter methodsFor: 'initalizing' stamp: 'cwp 2/9/2004 20:54'!
setMetaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBFilter methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:56'!
monitor: aMonitor
	monitor _ aMonitor! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/9/2004 20:50'!
children
	^ metaNode children! !

!OBFilter methodsFor: 'accessing' stamp: 'cwp 2/11/2004 23:48'!
metaNode
	^ metaNode! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/29/2004 14:34'!
selectAncestorOf: aNode withParent: parentNode
	^ (metaNode nodesForParent: parentNode)
		detect: [:child | child isAncestorOf: aNode] 
		ifNone: [nil]! !

Object subclass: #OBMetaGraphBuilder
	instanceVariableNames: ''
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBMetaGraphBuilder commentStamp: 'cwp 3/5/2004 12:39' prior: 0!
This is a utility class for building and installing all the metagraphs used by the standard browsers.!

!OBMetaGraphBuilder class methodsFor: 'instance creation' stamp: 'cwp 3/3/2004 22:54'!
default
	^ Instance ifNil: [Instance _ self new]! !

Object subclass: #OBActor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBActor commentStamp: 'cwp 3/4/2004 21:02' prior: 0!
Actors the basic units of domain-related functionality. They are attached to metanodes, and supply the Actions used to build context menus and buttons in the browser. Actors can be proxies for functionality supplied by other classes, as in OBFileRegistryActor, or they may supply the funtionality them selves. 

OBActor is an abstract class which defines the protocol Actors must implement.Actors need not be subclasses of OBActor, however.!

!OBActor methodsFor: 'public' stamp: 'cwp 2/4/2004 15:25'!
actionsForNode: aNode
	^#()! !

!OBActor methodsFor: 'accessing' stamp: 'cwp 9/16/2004 23:25'!
actionWithLabel: aString selector: aSymbol 
	self shouldBeImplemented! !

OBActor subclass: #OBNodeActor
	instanceVariableNames: 'nodeClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Actors'!

!OBNodeActor commentStamp: 'cwp 3/5/2004 12:23' prior: 0!
For simplicity, many actions are implemented by the nodes themselves. When the contextual menu for a node is displayed, this Actor gathers those actions and makes them available in to the user.

If a node class is specified, the OBNodeActor will also make available any class side actions provided by the node class. This is convenient for the user, since it makes it possible to invoke an action without first selecting a node. Many 'create' actions are implemented in this way.

iVars:

nodeClass - if not nil, this class's class-side actions will be available to the user as well.!

!OBNodeActor methodsFor: 'public' stamp: 'cwp 3/3/2004 22:46'!
actionsForParent: aNode
	^ nodeClass 
		ifNotNil: [nodeClass actionsForParent: aNode]
		ifNil: [#()]! !

!OBNodeActor methodsFor: 'public' stamp: 'cwp 3/3/2004 22:42'!
nodeClass: aClass
	nodeClass _ aClass! !

!OBNodeActor class methodsFor: 'as yet unclassified' stamp: 'cwp 3/3/2004 22:51'!
onNodeClass: aClass
	^ self new nodeClass: aClass! !

!OBNodeActor methodsFor: 'public' stamp: 'cwp 1/23/2004 23:24'!
actionsForNode: aNode
	^ aNode actions! !

!OBActor methodsFor: 'public' stamp: 'cwp 2/5/2004 20:12'!
actionsForParent: aNode
	^#()! !

Object subclass: #OBColumn
	instanceVariableNames: 'panel filter parent children selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBColumn commentStamp: 'cwp 3/4/2004 21:48' prior: 0!
An OBColumn manages a list of nodes, which it displays in a PluggableListMorph in the pane scroller at the top of the browser. It's main responsibility is keeping its list - and those of its neighbours - up to date. Each column has a MetaNode, which provides the list contents. It uses a filter to meditate between it and the MetaNode.

iVars:

browser		- the browser which owns the column
filter		- the filter which manages the column's MetaNode.
parent		- the node selected in the column to the left of this column
children 	- the nodes which make up this column's list
selection 	- the index of the node selected by the user!

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 18:47'!
filter: aFilter
	filter _ aFilter monitor: self.
	self changed: #filter.! !

!OBColumn methodsFor: 'updating' stamp: 'avi 3/6/2004 16:22'!
clear
	self clearNext.
	filter _ nil.
	parent _ nil.
	children _ #().
	self clearSelection.
	self changed: #list.
	self changed: #selection.
	self changed: #filter.! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/9/2004 21:06'!
columnWithHeader
	| col bh |
	col _ BorderedMorph new
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		clipSubmorphs: true;
		color: Color transparent;
		cellInset: 0;
		borderWidth: 0;
		layoutPolicy: ProportionalLayout new.
	
	bh _ self buttonHeight negated.
	col
		addMorph: self listMorph
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1) offsets: (0@0 corner: 0@bh)).
	col
		addMorph: self filter buttonMorph
		fullFrame: (LayoutFrame fractions: (0@1 corner: 1@1) offsets: (0@bh corner: 0@0)).
		

	^ col! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 2/26/2004 22:06'!
getChildren
	children _ self filter nodesForParent: parent.
	^ children! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/12/2004 18:10'!
simplePane
	^ OBPane forColumn: self! !

!OBColumn methodsFor: 'updating' stamp: 'avi 3/6/2004 23:11'!
nextChanged
	self hasSelection ifTrue:
		[self next ifNotNilDo:
			[:next |
			next
				metaNode: self nextMetaNode 
				parent: self selectedNode]]! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 23:07'!
hasSelection
	^ self selection > 0! !

!OBColumn class methodsFor: 'instance creation' stamp: 'cwp 11/16/2004 21:57'!
inPanel: aBrowser
	^ self new setPanel: aBrowser! !

!OBColumn class methodsFor: 'instance creation' stamp: 'cwp 11/16/2004 21:58'!
inPanel: aPanel metaNode: aMetaNode node: aNode
	^ self new
			setPanel: aPanel
			metaNode: aMetaNode
			node: aNode! !

!OBColumn methodsFor: 'printing' stamp: 'cwp 2/11/2004 23:31'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self descriptor.
	aStream nextPut: $)! !

!OBColumn methodsFor: 'callbacks' stamp: 'hpt 5/17/2004 14:20'!
keystroke: aChar from: aMorph
	self isEmpty ifFalse: [(self actionsForKeystroke: aChar)
									do: [:action | 
										action monitor: self.
										action trigger]]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/20/2004 22:04'!
okToChange
	^panel basicOkToChange! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:49'!
basicNext
	^panel columnAfter: self! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/16/2004 21:49'!
deselected
	panel deselected: self.
	self clearNext! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:36'!
dragTransferTypeForMorph: listMorph
	^ self dragTransferType! !

!OBColumn methodsFor: 'selecting' stamp: 'avi 3/6/2004 18:08'!
select: aNode
	self selection: (children indexOf: (children detect: [:ea | ea name = aNode name] ifNone: []))! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 16:27'!
shouldBeLast
	^ self hasSelection not or: [self nextMetaNode hasChildren not]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/16/2004 21:49'!
nodeDeleted: aNode 
	"This gets called by OBNodeDelete if an action causes the currently selected to be deleted."

	panel nodeDeleted: aNode.
	self isEmpty 
		ifFalse: 
			[self getChildren.
			self changed: #list.
			self selection: 0]! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 11/20/2004 21:24'!
selectionChanged
	self clicked.
	self changed: #selection.
	panel basicNodeChanged! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 2/25/2004 21:26'!
wantsDroppedMorph: transferMorph event: evt inMorph: listMorph
	| node |
	(transferMorph isKindOf: TransferMorph) ifFalse: [^ false].
	transferMorph dragTransferType == self dragTransferType ifFalse: [^ false].
	node _ self nodeForDroppedMorph: transferMorph event: evt inMorph: listMorph.
	^ node notNil and: [node wantsDroppedNode: transferMorph passenger]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/16/2004 23:22'!
selected
	self next 
		ifNotNilDo: [:column | column metaNode: self nextMetaNode parent: self selectedNode].
	panel selected: self! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 20:50'!
next
	^ self basicNext
		ifNil: [(self hasSelection and: [self shouldBeLast not])
				ifTrue: [self createNext]]
	! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 5/13/2004 10:45'!
notification: aNotification
	aNotification notify: self.! !

!OBColumn methodsFor: 'actions' stamp: 'avi 3/6/2004 16:26'!
actionsForSelectedNode
	| node |
	node _ self selectedNode ifNil: [^ #()].
	^ node metaNode actionsForNode: node	! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 1/24/2004 01:03'!
list
	^ children 
		ifNil: [#()]
		ifNotNil: [children collect: [:ea | ea name]]! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/20/2004 21:24'!
parent: aNode 
	parent := aNode.
	self getChildren.
	self clearSelection.
	self changed: #list.
	self nextChanged.
	panel basicNodeChanged! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/16/2004 21:49'!
nodeSelected: aNode 
	panel okToChange ifTrue: [panel jumpTo: aNode]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 2/29/2004 21:48'!
listSize
	^ children
		ifNil: [0]
		ifNotNil: [children size]! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/11/2004 23:22'!
isEmpty
	^ parent isNil! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/7/2004 15:56'!
parent
	^ parent! !

!OBColumn methodsFor: 'selecting' stamp: 'avi 3/6/2004 18:05'!
clearSelection
	selection _ 0! !

!OBColumn methodsFor: 'initializing' stamp: 'cwp 11/16/2004 21:57'!
setPanel: aPanel 
	panel := aPanel.
	children := #().
	self clearSelection! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 19:53'!
metaNode: aMetaNode parent: aNode
	self filter: aMetaNode filter.
	self parent: aNode.
	self clearNext.! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/17/2004 00:18'!
browser
	^ panel browser! !

!OBColumn methodsFor: 'actions' stamp: 'cwp 9/14/2004 19:08'!
actionSetsForSelectedNode
	| node |
	node _ self selectedNode ifNil: [^ #()].
	^ node metaNode actionSetsForNode: node	! !

!OBColumn methodsFor: 'actions' stamp: 'cwp 2/9/2004 20:45'!
actionsForParentNode
	^ self filter children gather: [:child | child actionsForParent: parent].	! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:58'!
filter
	^ filter! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 11/27/2004 01:36'!
listMorph
	^ (PluggableListMorph
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:
		menu: #menu:
		keystroke: #keystroke:from:)
			getListElementSelector: #listAt:;
			getListSizeSelector: #listSize;
			dragEnabled: self dragEnabled;
			dropEnabled: self dropEnabled;
			borderWidth: 0;
			autoDeselect: false;
			adoptPaneColor: panel defaultBackgroundColor;
			yourself! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:35'!
dragPassengerFor: item inMorph: listMorph
	^ self nodeForItem: item inMorph: listMorph! !

!OBColumn methodsFor: 'updating' stamp: 'avi 3/6/2004 16:26'!
clicked
	self hasSelection
		ifFalse: [self deselected]
		ifTrue: [self selected]! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 3/14/2004 15:18'!
selectSilently: aNode
	selection _ (children indexOf: (children detect: [:ea | ea name = aNode name] ifNone: []))! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/12/2004 18:11'!
paneWithHeader
	^ OBPane forColumn: self withFilter: self filter! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:51'!
panel: aBrowser 
	panel := aBrowser! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 9/14/2004 19:11'!
menu: aMenu
	self isEmpty ifFalse: [self actionSetsForParentNode, self actionSetsForSelectedNode
							do: [:set |set do: [:action | 
											action monitor: self.
											action addItemToMenu: aMenu]]
							separatedBy: [aMenu addLine]].
	^ aMenu! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/20/2004 21:24'!
refresh
	| node oldChildren |
	self isEmpty ifTrue: [^self].
	node := self selectedNode.
	oldChildren := children.
	self getChildren.
	children = oldChildren 
		ifFalse: 
			[self selectSilently: node.
			self hasSelection 
				ifFalse: 
					[self clearNext.
					panel basicNodeChanged]].
	self changed: #list! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 13:45'!
dragEnabled
	^ true! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/16/2004 23:16'!
createNext
	| nextMetaNode created |
	nextMetaNode := self nextMetaNode.
	created := nextMetaNode columnInPanel: panel node: self selectedNode.
	panel pushColumn: created.
	^created! !

!OBColumn methodsFor: 'initializing' stamp: 'cwp 11/16/2004 21:57'!
setPanel: aPanel metaNode: aMetaNode node: aNode 
	panel := aPanel.
	filter := aMetaNode filter monitor: self.
	parent := aNode.
	self getChildren.
	self clearSelection! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 2/12/2004 20:50'!
clearNext
	self basicNext ifNotNilDo: [:next | next clear].! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 2/25/2004 22:25'!
acceptDroppingMorph: transferMorph event: evt inMorph: listMorph
	^ [(self nodeForDroppedMorph: transferMorph event: evt inMorph: listMorph)
		acceptDroppedNode: transferMorph passenger]
			on: OBNotification
			do: [:aNotification |
				self notification: aNotification.
				aNotification resume]
! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/12/2004 19:17'!
morph
	^ self isEmpty 
		ifTrue: [self simplePane]
		ifFalse: [self filter wantsButton
							ifTrue: [self paneWithHeader]
							ifFalse: [self simplePane]]! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 2/25/2004 22:18'!
nodeForItem: item inMorph: pluggableListMorph
	^ children 
		detect: [:child | child name = item contents]
		ifNone: [item contents]! !

!OBColumn methodsFor: 'actions' stamp: 'hpt 5/17/2004 14:19'!
actionsForKeystroke: aChar
	^self actionsForParentNode, self actionsForSelectedNode
		select: [:anAction | anAction keystroke == aChar]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 3/21/2004 21:35'!
listAt: index
	^ (children at: index ifAbsent: [^ '']) name! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:18'!
wantsButton
	^ self filter notNil and: [self filter wantsButton]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/16/2004 21:52'!
definitionChanged: aDefinition 
	(self browser panels detect: [:ea | ea isDefinitionPanel] ifNone: [^self]) 
		definition: aDefinition! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 11/5/2003 19:01'!
selection
	^ selection ifNil: [0]! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:59'!
metaNode
	^ self filter metaNode! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 2/2/2004 21:54'!
selectedNode
	^ children at: self selection ifAbsent: [].! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 12/14/2003 22:15'!
buttonHeight
	^ 20! !

!OBColumn methodsFor: 'printing' stamp: 'avi 3/6/2004 16:23'!
descriptor
	self isEmpty ifTrue: [^ 'empty'].
	self hasSelection ifTrue: [^ self selectedNode name].
	^ ''! !

!OBColumn methodsFor: 'actions' stamp: 'cwp 9/14/2004 19:09'!
actionSetsForParentNode
	^ self filter children gather: [:child | child actionSetsForParent: parent].	! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 1/30/2004 00:38'!
accept: aText notifying: aController
	self selectedNode accept: aText notifying: aController.! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:11'!
nextMetaNode
	^ self selectedNode metaNode! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 5/10/2004 14:19'!
selectAncestorOf: aNode
	| ancestor |
	ancestor _ self filter selectAncestorOf: aNode withParent: parent.
	ancestor ifNotNil: [self 
						getChildren; 
						changed: #list;
						select: ancestor; 
						selected].
	^ ancestor! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:30'!
dropEnabled
	^ true! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 11/20/2004 21:24'!
listChanged
	self refresh.
	self nextChanged.
	panel basicNodeChanged! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 3/3/2004 02:23'!
nodeForDroppedMorph: transferMorph event: evt inMorph: pluggableListMorph
	| index item |
	index _ pluggableListMorph rowAtLocation: evt position.
	index = 0 ifTrue: [^ nil].
	item _ pluggableListMorph listMorph item: index.
	^ self nodeForItem: item inMorph: pluggableListMorph! !

!OBColumn methodsFor: 'selecting' stamp: 'avi 3/6/2004 16:22'!
selection: anInteger
	selection _ anInteger.
	self selectionChanged.
! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:37'!
dragTransferType
	^ #OmniBrowser! !

TestCase subclass: #OBRadioButtonBarTest
	instanceVariableNames: 'bar selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 2/24/2004 19:15'!
testUpdateSelection
	self changed: #list.
	selection _ 2.
	self changed: #selection.
	self assertSelectionIs: 2.! !

!OBRadioButtonBarTest methodsFor: 'callbacks' stamp: 'cwp 2/24/2004 19:31'!
selection: anInteger
	selection _ anInteger.
	self changed: #selection! !

!OBRadioButtonBarTest methodsFor: 'simulating' stamp: 'cwp 11/27/2004 18:04'!
push: aString
	| button |
	button _ bar submorphs detect: [:b | b label contents = aString].
	button performAction! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 11/27/2004 18:11'!
testLayout
	| rects |
	bar bounds: (10@10 corner: 310@33).
	self changed: #list.
	
	"This is an ugly hack. I can't figure out how else to force a layout."
	bar openInWorld.
	rects _ bar submorphs collect: [:b | b bounds].
	bar delete.

	rects do: [:ea | self assert: ea top = 10; assert: ea bottom = 30].
	self assert: rects first left = 10.
	self assert: rects first right = rects second left.
	self assert: rects second right = rects third left.
	self assert: rects third right = 310.! !

!OBRadioButtonBarTest methodsFor: 'callbacks' stamp: 'cwp 2/22/2004 17:42'!
list
	^ #('alpha' 'beta' 'gamma')! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 11/27/2004 18:04'!
testUpdateList
	| labels |
	self changed: #list.
	labels _ bar submorphs collect: [:b | b label contents].
	self assert: labels size = self list size.
	self assert: (labels includesAllOf: self list)! !

!OBRadioButtonBarTest methodsFor: 'asserting' stamp: 'cwp 11/27/2004 18:03'!
assertSelectionIs: anInteger
	self assert: selection = anInteger.
	bar submorphs withIndexDo: 
		[:button :index |
		index = anInteger
			ifTrue: [self assert: button isOn]
			ifFalse: [self deny: button isOn]]! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 2/24/2004 19:35'!
testPushAlphaBeta
	self changed: #list.
	self push: 'alpha'.
	self assertSelectionIs: 1.
	self push: 'beta'.
	self assertSelectionIs: 2.! !

!OBRadioButtonBarTest methodsFor: 'running' stamp: 'cwp 11/27/2004 18:03'!
setUp
	selection _ 0.
	bar _ OBRadioButtonBar
			on: self
			list: #list
			selected: #selection
			changeSelected: #selection:.
! !

!OBRadioButtonBarTest methodsFor: 'testing' stamp: 'cwp 2/22/2004 16:53'!
testList
	self assert: bar list = self list! !

!OBRadioButtonBarTest methodsFor: 'callbacks' stamp: 'cwp 2/22/2004 16:39'!
selection
	^ selection! !

TestCase subclass: #OBPanelLayoutTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 12/5/2004 18:28'!
testTextAndButtonPanels
	| window |
	window _ self windowWithPanels: {OBColumnPanel. OBFixedButtonPanel. OBDefinitionPanel}.
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs third layoutFrame
		hasFractions: (0@0.4 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@25).
	self
		assertFrame: window submorphs fourth layoutFrame
		hasFractions: (0@0.4 corner: 1@1)
		andOffsets: (0@25 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 11/7/2004 21:27'!
metaNode
	^ OBFakeNode metagraph! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 12/5/2004 18:28'!
testNavPanel
	| window |
	window _ self windowWithPanels: {OBColumnPanel}.
	self assert: window submorphs size = 2.
	self assert: window submorphs second knownName = 'scroller'.
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@1)
		andOffsets: (0@0 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 12/5/2004 18:24'!
windowWithPanels: panelClasses
	| browser panels |
	panels := panelClasses collect: [:ea | ea new].
	browser _ OBBrowser 
				metaNode: self metaNode 
				root: self node
				selection: nil
				panels: panels. 
	^ browser morph
! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 12/5/2004 18:29'!
testTwoTextPanels
	"Yeah, ok, this is a bit ridiculous"
	
	| window |
	window _ self windowWithPanels: {OBColumnPanel. OBDefinitionPanel. OBDefinitionPanel}.	
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs third layoutFrame
		hasFractions: (0@0.4 corner: 1@0.7)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs fourth layoutFrame
		hasFractions: (0@0.7 corner: 1@1)
		andOffsets: (0@0 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'testing' stamp: 'cwp 12/5/2004 18:29'!
testTextPanel
	| window |
	window _ self windowWithPanels: {OBColumnPanel. OBDefinitionPanel}.
	self assert: window submorphs size = 3.
	self assert: window submorphs second knownName = 'scroller'.
	self
		assertFrame: window submorphs second layoutFrame
		hasFractions: (0@0 corner: 1@0.4)
		andOffsets: (0@0 corner: 0@0).
	self
		assertFrame: window submorphs third layoutFrame
		hasFractions: (0@0.4 corner: 1@1)
		andOffsets: (0@0 corner: 0@0)! !

!OBPanelLayoutTest methodsFor: 'accessing' stamp: 'cwp 8/25/2004 21:44'!
node
	^ OBFakeNode new! !

!OBPanelLayoutTest methodsFor: 'asserting' stamp: 'cwp 3/9/2004 22:21'!
assertFrame: frame hasFractions: fraction andOffsets: offset
	self assert: frame leftFraction = fraction left.
	self assert: frame topFraction = fraction top.
	self assert: frame rightFraction = fraction right.
	self assert: frame bottomFraction = fraction bottom.
	
	self assert: frame leftOffset = offset left.
	self assert: frame topOffset = offset top.
	self assert: frame rightOffset = offset right.
	self assert: frame bottomOffset = offset bottom.
! !

TestCase subclass: #OBBrowserTest
	instanceVariableNames: 'model morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/11/2004 23:02'!
clickButtonLabeled: aString
	| button |
	button _ (self morphsOfClass: PluggableButtonMorph) 
				detect: [:m | m label contents = aString]
				ifNone: [self signalFailure: 'No button labeled ', aString].
	button performAction! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/16/2004 20:33'!
assertTextIs: aString
	| actual |
	actual _ self textMorph text.
	self assert: actual = aString.! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 1/30/2004 01:20'!
assertCurrentItemIs: aString
	self assert: self currentItem = aString! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/11/2004 23:03'!
assertButtonSelected: aString
	| button |
	button _ (self morphsOfClass: PluggableButtonMorph) 
				detect: [:m | m label contents = aString]
				ifNone: [self signalFailure: 'No button labeled ', aString].
	self assert: button isOn! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:10'!
assertAListIncludes: anArrayOfStrings
	self listMorphs 
			detect: [:m | m getListDelicately includesAllOf: anArrayOfStrings]
			ifNone: [self assert: false].! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:50'!
drop: aPassenger on: targetName
	| list transfer event |
	list _ self findListContaining: targetName.
	self assert: list notNil.
	transfer _ TransferMorph withPassenger: aPassenger.
	transfer dragTransferType: #OmniBrowser.
	event _ self mouseMoveEventIn: targetName.
	self assert: (list model wantsDroppedMorph: transfer event: event inMorph: list).
	^ list model acceptDroppingMorph: transfer event: event inMorph: list! !

OBBrowserTest subclass: #OBFakeBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 14:25'!
testAcceptText
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).
	self assertTextIs: 'a'.
	self setText: 'x'.
	
	"This is a gratuitous refresh to make the test pass. It isn't necessary when the 
	browser is actually open. It's a deficiency of the test environment..."
	self listMorphs do: [:ea | ea update: #list].
	
	self assertListAt: 1 contains: #(x b c).
	self assertListAt: 2 contains: #(xa xb xc).
	self assertTextIs: 'x'
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/7/2003 19:30'!
testSelectC
	self select: #c.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #().
	self assertTextIs: 'c'! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 12/7/2003 19:30'!
testSelectA
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).
	self assertTextIs: 'a'! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 15:16'!
testRefreshDirty
	| root |
	self select: #a.
	self editText: 'x'.
	
	root _ model root.
	root adopt: (OBFakeNode parent: root item: 'z').
	model refresh.
	
	self assertListAt: 1 contains: #(a b c z).
	self assertCurrentItemIs: #a.
	self assertTextIs: 'x'.
! !

!OBFakeBrowserTest methodsFor: 'constants' stamp: 'cwp 12/5/2004 18:17'!
paneCount
	^ 4! !

!OBFakeBrowserTest methodsFor: 'running' stamp: 'cwp 11/7/2004 21:27'!
metagraph
	^ OBFakeNode metagraph! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:00'!
testPanes
	| scroller |
	self 
		assert: (self morphsOfClass: PluggableListMorph) size = self paneCount.
	self assert: (self morphsOfClass: OBPluggableTextMorph) size = 1.
	scroller := self morphNamed: 'scroller'.
	self assert: scroller panes size = model navigationPanel minPanes! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 13:49'!
testConfirmAbandon
	self select: #a.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(aa ab ac).
	self assertTextIs: 'a'.
	self editText: 'x'.
	[self select: #b]
		on: OBConfirmationRequest
		do: [:notification | notification cancel].
	self assertCurrentItemIs: #a.
	self assertTextIs: 'x'.
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 11/16/2004 22:42'!
testDragAndDrop
	| node |
	self select: #a.
	node _ self drag: #ab.
	self assert: (node isKindOf: OBFakeNode).
	self assert: node name = #ab.
	self drop: node on: #c.
	self assertListAt: 1 contains: #(a b c).
	self assertListAt: 2 contains: #(cb).
	self assertCurrentItemIs: #c.
	self assertTextIs: 'c'! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 8/26/2004 02:17'!
testInitialDisplay
	self assert: self listMorphs size = self paneCount.
	self assertListAt: 1 contains: #(a b c).
	self assertTextIs: ''.
	! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 17:20'!
testDeleteRootNode
	self select: #a.
	self selectMenuItem: 'delete root'.
	self denyAListIncludes: #(a b c).
	self assertTextIs: ''! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 8/26/2004 02:17'!
testDeleteB
	self select: #b.
	self selectMenuItem: 'delete'.
	self assertListAt: 1 contains: #(a c).
	self assert: self listMorphs size = self paneCount.
	self assertTextIs: ''.! !

!OBFakeBrowserTest methodsFor: 'constants' stamp: 'cwp 12/5/2004 18:18'!
browserClass
	^ OBBrowser! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 8/26/2004 02:17'!
testDeselectA
	self select: #a.
	self deselect: #a.
	self assertListAt: 1 contains: #(a b c).
	self assert: self listMorphs size = self paneCount.
	self assertTextIs: ''.! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 11/23/2003 13:12'!
testCreateWindow
	self assert: (morph isKindOf: SystemWindow).
	self assert: (morph label = 'OmniBrowser').
! !

!OBFakeBrowserTest methodsFor: 'testing' stamp: 'cwp 1/7/2004 08:27'!
testAction
	self select: #a.
	self selectMenuItem:  'become z'.
	self assertListAt: 1 contains:  #(z b c)! !

!OBFakeBrowserTest methodsFor: 'running' stamp: 'cwp 12/17/2003 15:12'!
node
	^ OBFakeNode new! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:10'!
assertAListMatches: strings
	| list |
	self listMorphs
			detect:	[:m | 
					list _ m getListDelicately.
					(list size = strings size) and: [list includesAllOf: strings]]
			ifNone: [self assert: false].! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 1/23/2004 23:30'!
selectMenuItem: aString
	| item |
	item _ self menuItemNamed: aString.
	item ifNil: [self signalFailure: 'Menu item ''', aString, ''' does not exist.'].
	item doButtonAction! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
morphsOfClass: aClass
	| found |
	found _ OrderedCollection new.
	morph allMorphsDo: [:m | (m isMemberOf: aClass) ifTrue: [found add: m]].
	^ found! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 3/14/2004 13:11'!
findListContaining: aString
	^ self listMorphs detect: [:m | m getListDelicately includes: aString] ifNone: []! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 2/29/2004 19:38'!
assertSelectedTextIs: aString
	| text selection |
	morph _ self textMorph.
	text _ morph getText.
	selection _ morph getSelection.
	self assert: (text atAll: selection) asString = aString.! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:50'!
drag: passangerName to: targetName
	| passanger |
	passanger _ self drag: passangerName.
	self drop: passanger on: targetName! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 11/17/2004 23:45'!
forceLayout
	morph bounds: (6@6 corner: 606@406)! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
morphNamed: aString
	^ morph 
		findDeepSubmorphThat: [:m | m knownName = aString]
		ifAbsent: [self signalFailure: aString, 'not found']! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 12/17/2003 15:08'!
setText: aString
	| textMorph |
	textMorph _ self textMorph.
	textMorph 
		selectAll;
		replaceSelectionWith: aString asText;
		accept.
	! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 1/23/2004 22:35'!
assertMenuContains: aString
	| item |
	item _ self menuItemNamed: aString.
	self assert: item notNil! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 2/29/2004 19:33'!
assertTextIsSelected
	self assert: self textMorph getSelection size = self textMorph getText size! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 5/13/2004 10:59'!
currentList
	^ self listMorphs reversed 
						detect: [:list | list selection notNil]
						ifNone: [self signalFailure: 'No item is currently selected']! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/14/2004 13:46'!
select: aString
	"We don't use #getListDelicately because the user would have to scroll through the list
	to find the item we're selecting, thus triggering updates to the list."
	
	| listMorph |
	listMorph _ self findListContaining: aString.
	self assert: listMorph notNil.
	listMorph model okToChange 
		ifTrue: [listMorph changeModelSelection: (listMorph getListObtrusively indexOf: aString)].! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/14/2004 13:14'!
mouseMoveEventIn: name
	| rect plm |
	plm _ self findListContaining: name.
	rect _ plm listMorph rectForRow: (plm getListDelicately indexOf: name).
	^ MouseMoveEvent new
		setType: #mouseEnter
		startPoint: rect center + (rect width @ 0)
		endPoint: rect center
		trail: #()
		buttons: 4
		hand: ActiveHand
		stamp: Time millisecondClockValue! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:50'!
drag: nodeName
	| item list |
	item _ StringMorph contents: nodeName asString.
	list _ self findListContaining: nodeName.
	^ list model dragPassengerFor: item inMorph: list! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:11'!
denyAListIncludes: anArrayOfStrings
	self listMorphs 
			detect: [:m | m getListDelicately includesAllOf: anArrayOfStrings]
			ifNone: [^ self].
	self assert: false.! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 12/17/2003 15:08'!
deselect: aString
	| listMorph |
	listMorph _ self findListContaining: aString.
	listMorph changeModelSelection: 0.! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 12/17/2003 15:08'!
listMorphs
	^ self morphsOfClass: PluggableListMorph! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 3/14/2004 13:11'!
listItemsAt: index
	^ (self listMorphs 
		at: index 
		ifAbsent: [self signalFailure: 'List ', index asString, ' does not exist'])
			getListDelicately
	! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 12:47'!
assertListAt: index contains: items
	| list |
	list _ self listItemsAt: index.
	self assert: (list size = items size).
	self assert: (list includesAllOf: items).! !

!OBBrowserTest methodsFor: 'running' stamp: 'cwp 12/5/2004 18:16'!
setUp
	model _ self browserClass metaNode: self metagraph node: self node.
	morph _ model morph.
	self forceLayout! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 2/12/2004 20:39'!
assertNoButtonsVisible
	self assert: (self morphsOfClass: PluggableButtonMorph) isEmpty.! !

!OBBrowserTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/17/2003 15:10'!
isAbstract
	^ self name = 'OBBrowserTest'! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 1/31/2004 13:23'!
currentItem
	^ self currentList selection! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 2/28/2004 13:52'!
editText: aString
	| textMorph |
	textMorph _ self textMorph.
	textMorph 
		selectAll;
		replaceSelectionWith: aString asText! !

!OBBrowserTest methodsFor: 'simulating' stamp: 'cwp 3/14/2004 13:46'!
selectSequence: aCollection
	| plm |
	aCollection withIndexDo: [:item :index | 
					plm _ self listMorphs at: index.
					plm changeModelSelection: (plm getListObtrusively indexOf: item)]! !

!OBBrowserTest methodsFor: 'asserting' stamp: 'cwp 3/14/2004 13:01'!
assertListAt: index includes: items
	self assert: ((self listItemsAt: index) includesAllOf: items).! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 1/26/2004 21:53'!
menuItemNamed: aString
	| lists current |
	lists _ self listMorphs.
	current _ lists reversed detect: [:list | list selection notNil].
	^ (current getMenu: true) itemWithWording: aString.
! !

!OBBrowserTest methodsFor: 'examining' stamp: 'cwp 11/6/2004 23:00'!
textMorph
	^ (self morphsOfClass: OBPluggableTextMorph) last! !

TestCase subclass: #OBPaneScrollerTest
	instanceVariableNames: 'paneCount scroller'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPaneScrollerTest methodsFor: 'actions' stamp: 'cwp 11/21/2004 22:40'!
createScroller
	scroller := OBPaneScroller withModel: self.
	scroller bounds: (10 @ 420 extent: 600 @ 160)! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/21/2004 22:40'!
testPaneHeightWithScrollBar
	paneCount := 5.
	self createScroller.
	self assert: scroller paneHeight = 147! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/21/2004 23:09'!
testResizeUpdatesPaneWidth
	paneCount := 4.
	self createScroller.
	self assert: scroller panes first width = 147.
	scroller bounds: (scroller bounds outsetBy: 2).
	self assert: scroller panes first width = 148! !

!OBPaneScrollerTest methodsFor: 'callbacks' stamp: 'cwp 11/17/2004 23:39'!
sizing
	^ 4! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/23/2004 01:38'!
testPaneReclamation
	paneCount := 6.
	self createScroller.
	scroller submorphs first setValue: 0.2.
	scroller step.
	self assert: scroller panes size = 4! !

!OBPaneScrollerTest methodsFor: 'actions' stamp: 'cwp 11/2/2004 00:00'!
createPane
	^ RectangleMorph new
		bounds: (10@10 corner: 20@20);
		yourself! !

!OBPaneScrollerTest methodsFor: 'callbacks' stamp: 'cwp 11/21/2004 12:55'!
panes
	^ ((1 to: paneCount) collect: [:i | self createPane]) asOrderedCollection! !

!OBPaneScrollerTest methodsFor: 'callbacks' stamp: 'cwp 11/23/2004 01:37'!
reclaimPanes
	^ scroller panes size - 4! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/21/2004 23:11'!
testResizeUpdatesPaneHeight
	paneCount := 4.
	self createScroller.
	self assert: scroller panes first height = 160.
	scroller bounds: (scroller bounds outsetBy: 1).
	self assert: scroller panes first height = 162! !

!OBPaneScrollerTest methodsFor: 'tests' stamp: 'cwp 11/21/2004 22:40'!
testPaneHeightNoScrollBar
	paneCount := 4.
	self createScroller.
	self assert: scroller paneHeight = 160! !

TestCase subclass: #OBOpenTest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBOpenTest methodsFor: 'actions' stamp: 'cwp 12/5/2004 15:54'!
create: aBlock
	self 
		shouldnt: [browser _ aBlock value]
		raise: OBBrowseRequest! !

!OBOpenTest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/5/2004 15:55'!
isAbstract
	^ self name = #OBOpenTest! !

OBOpenTest subclass: #OBLibraryOpenTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBLibraryOpenTest methodsFor: 'tests' stamp: 'cwp 12/5/2004 16:19'!
testGraphRootSelectionPanels
	| parent child panels |
	parent _ OBFakeNode parent: nil item: 'parent'.
	child _ OBFakeNode parent: parent item: 'a'.
	panels _ {OBColumnPanel new. OBDefinitionPanel new}.
	self create: [OBBrowser 
					graph: self metagraphName
					root: parent
					selection: child
					panels: panels].
	self assert: browser navigationPanel minPanes = 1.
	self assert: browser navigationPanel maxPanes = 1.
	self assert: browser root metaNode = (OBBrowser metagraphAt: self metagraphName).
	self assert: browser root = parent.
	self assert: browser currentNode = child.
	self assert: browser panels size = panels size.
	self assert: (browser panels includesAllOf: panels)! !

!OBLibraryOpenTest methodsFor: 'running' stamp: 'cwp 12/5/2004 16:06'!
tearDown
	OBBrowser removeMetagraphAt: self metagraphName! !

!OBLibraryOpenTest methodsFor: 'tests' stamp: 'cwp 12/5/2004 16:43'!
testGraphRootSelection
	| parent child |
	parent _ OBFakeNode parent: nil item: 'parent'.
	child _ OBFakeNode parent: parent item: 'a'.
	self create: [OBBrowser 
					graph: self metagraphName
					root: parent
					selection: child].
	self shouldnt: [browser navigationPanel] raise: Error! !

!OBLibraryOpenTest methodsFor: 'accessing' stamp: 'cwp 12/5/2004 16:07'!
metagraphName
	^ #Fake! !

!OBLibraryOpenTest methodsFor: 'running' stamp: 'cwp 12/5/2004 16:07'!
setUp
	OBBrowser metagraphAt: self metagraphName put: OBFakeNode metagraph! !

!OBOpenTest methodsFor: 'actions' stamp: 'cwp 12/5/2004 15:52'!
open: aBlock
	[aBlock value]
		on: OBBrowseRequest
		do: [:n | browser _ n browser].
	self assert: browser notNil! !

TestCase subclass: #OBPaneLayoutTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBPaneLayoutTest methodsFor: 'callbacks' stamp: 'cwp 11/2/2004 00:37'!
buttonMorph
	^ RectangleMorph new
		color: Color red;
		yourself! !

!OBPaneLayoutTest methodsFor: 'testing' stamp: 'cwp 11/7/2004 00:13'!
testButtonPlacement
	| pane button |
	pane := OBPane forColumn: self withFilter: self.
	pane bounds: (0@0 corner: 200@400).
	button := pane submorphs at: 1.
	self assert: button top = 377.
	self assert: button bottom = 400.
	self assert: button left = pane left.
	self assert: button right = pane right.! !

!OBPaneLayoutTest methodsFor: 'callbacks' stamp: 'cwp 11/2/2004 00:34'!
listMorph
	^ RectangleMorph new
! !

TestCase subclass: #OBButtonPanelTest
	instanceVariableNames: 'model morph pushed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 11:38'!
buttonLabels
	^ #(foo bar baz)! !

!OBButtonPanelTest methodsFor: 'accessing' stamp: 'cwp 11/27/2004 22:02'!
labelAt: index 
	^(morph submorphs at: index) submorphs first submorphs first! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 3/10/2004 23:08'!
currentNode
	^ (OBFakeNode new) metaNode: self metaNode! !

OBButtonPanelTest subclass: #OBVarButtonPanelTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBVarButtonPanelTest methodsFor: 'tests' stamp: 'cwp 8/28/2004 20:25'!
testButtons
	model nodeChanged.
	self assert: morph submorphs size = 3.
	#(foo bar baz) 
		with: morph submorphs 
		do: [:sel :m | self assert: m label contents = sel asString]! !

!OBVarButtonPanelTest methodsFor: 'running' stamp: 'cwp 8/28/2004 20:38'!
panel
	^ OBVarButtonPanel inBrowser: self.! !

!OBButtonPanelTest methodsFor: 'asserting' stamp: 'cwp 11/27/2004 22:04'!
denyButtonEnabled: index 
	self deny: (self labelIsBlack: index)! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 11:14'!
actionClass
	^ OBAction! !

!OBButtonPanelTest methodsFor: 'testing' stamp: 'cwp 5/27/2004 11:30'!
testLayout
	| rects |
	morph bounds: (10@10 corner: 310@33).
	model nodeChanged.
	
	"This is an ugly hack. I can't figure out how else to force a layout."
	morph openInWorld.
	rects _ morph submorphs collect: [:b | b bounds].
	morph delete.

	rects do: [:ea | self assert: ea top = 10; assert: ea bottom = 33].
	self assert: rects first left = 10.
	self assert: rects first right = rects second left.
	self assert: rects second right = rects third left.
	self assert: rects third right = 310.! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 11:40'!
actions
	^ self buttonLabels collect: [:ea | self actionWithLabel: ea]
! !

!OBButtonPanelTest methodsFor: 'callbacks' stamp: 'cwp 11/27/2004 21:52'!
defaultBackgroundColor
	^ Color red! !

!OBButtonPanelTest methodsFor: 'asserting' stamp: 'cwp 11/27/2004 22:04'!
assertButtonEnabled: index 
	self assert: (self labelIsBlack: index)! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 11:13'!
actionsForNode: aNode
	^ self actions! !

!OBButtonPanelTest class methodsFor: 'as yet unclassified' stamp: 'cwp 8/28/2004 20:23'!
isAbstract
	^ self name = #OBButtonPanelTest! !

!OBButtonPanelTest methodsFor: 'callbacks' stamp: 'cwp 3/10/2004 23:59'!
push: aSymbol
	pushed _ aSymbol! !

!OBButtonPanelTest methodsFor: 'testing' stamp: 'cwp 11/27/2004 22:04'!
labelIsBlack: index 
	^(self labelAt: index) color = Color black! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 3/10/2004 23:09'!
metaNode
	^ OBMetaNode new addActor: self! !

!OBButtonPanelTest methodsFor: 'running' stamp: 'cwp 8/28/2004 20:28'!
setUp
	model _ self panel.
	morph _ model morph.
! !

!OBButtonPanelTest methodsFor: 'emulating' stamp: 'cwp 8/29/2004 13:49'!
actionWithLabel: label
	^ OBAction
		label: label asString, 'menu version'
		buttonLabel: label asString
		receiver: self
		selector: #push:
		arguments: (Array with: label)! !

!OBButtonPanelTest methodsFor: 'testing' stamp: 'cwp 3/10/2004 00:01'!
testPushFoo
	model nodeChanged.
	morph submorphs first performAction.
	self assert: pushed = #foo
	! !

OBButtonPanelTest subclass: #OBFixedButtonPanelTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBFixedButtonPanelTest methodsFor: 'running' stamp: 'cwp 8/29/2004 11:42'!
panel
	| panel |
	panel _ OBFixedButtonPanel inBrowser: self.
	self buttonLabels do: [:ea | panel addButtonWithLabel: ea].
	^ panel
	! !

!OBFixedButtonPanelTest methodsFor: 'tests' stamp: 'cwp 11/27/2004 21:58'!
testEnabling
	model nodeChanged.
	self assertButtonEnabled: 1.
	self denyButtonEnabled: 2.
	self assertButtonEnabled: 3.! !

!OBFixedButtonPanelTest methodsFor: 'tests' stamp: 'cwp 8/29/2004 11:44'!
testPushBar
	model nodeChanged.
	morph submorphs second performAction.
	self assert: pushed isNil
	! !

!OBFixedButtonPanelTest methodsFor: 'running' stamp: 'cwp 8/29/2004 11:43'!
actions
	^ (self buttonLabels copyWithout: #bar) collect: [:ea | self actionWithLabel: ea]! !

TestCase subclass: #OBDefinitionPanelTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBDefinitionPanelTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 01:30'!
testBrowseItWithClass
	self
		assertPanel: [:p | p browseIt: #BogusA]
		browser: [:browser | browser currentNode theClass = BogusA]! !

!OBDefinitionPanelTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 01:35'!
testImpementorsOfIt
	self
		assertPanel: [:p | p implementorsOfIt: #testBrowseItWithSelector]
		browser: [:browser | browser root selector = #testBrowseItWithSelector]! !

!OBDefinitionPanelTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 01:36'!
testSendersOfIt
	self
		assertPanel: [:p | p sendersOfIt: #testBrowseItWithSelector]
		browser: [:browser | browser root selector = #testBrowseItWithSelector]! !

!OBDefinitionPanelTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 01:43'!
testReferencesToIt
	self
		assertPanel: [:p | p referencesToIt: #BogusA]
		browser: [:browser | browser root theClass = BogusA]! !

!OBDefinitionPanelTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 01:32'!
testBrowseItWithSelector
	self
		assertPanel: [:p | p browseIt: #testBrowseItWithSelector]
		browser: [:browser | browser root selector = #testBrowseItWithSelector]! !

!OBDefinitionPanelTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 01:31'!
testBrowseItWithGlobal
	self
		assertPanel: [:p | p browseIt: #Transcript]
		browser: [:browser | browser currentNode theClass = TranscriptStream]
! !

!OBDefinitionPanelTest methodsFor: 'asserting' stamp: 'cwp 10/31/2004 01:33'!
assertPanel: actionBlock browser: assertionBlock
	| notification |
	panel := OBDefinitionPanel inBrowser: self.
	[self assert: (actionBlock value: panel)]
		on: OBBrowseRequest
		do: [:ex | notification := ex].
	self deny: notification isNil.
	self assert: (assertionBlock value: notification browser)! !

TestCase subclass: #OBKeyBindingsTest
	instanceVariableNames: 'ptm message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:07'!
modifier: mod keycode: code
	| string |
	string := 'keystroke 988180 ', mod asString, ' ', code asString.
	^ KeyboardEvent readFrom: string readStream! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:08'!
ksCmdB
	^ self modifier: 64 keycode: 98! !

!OBKeyBindingsTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 00:35'!
testSendersOfIt
	self assert: self ksCmdN sends: #sendersOfIt:! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/30/2004 23:26'!
selection
	^ 1 to: 9! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/30/2004 23:57'!
thisMessage
	| context args |
	context := thisContext sender.
	args := Array new: context size.
	1 to: args size do: [:i | args at: i put: (context at: 1)].
	^ Message 
		selector: context method selector
		arguments: args ! !

!OBKeyBindingsTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 00:44'!
testReferencesToIt
	self assert: self ksCmdShiftN sends: #referencesToIt:! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/30/2004 23:26'!
text
	^ 'OBBrowser'! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/31/2004 00:25'!
sendersOfIt: aSelector
	message := self thisMessage.
	^ true! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/30/2004 23:58'!
browseIt: aSelector
	message _ self thisMessage.
	^ true! !

!OBKeyBindingsTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 00:35'!
testImplementorsOfIt
	self assert: self ksCmdM sends: #implementorsOfIt:! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:09'!
ksCmdShiftW
	^ self modifier: 72 keycode: 87! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/31/2004 00:39'!
implementorsOfIt: aSelector
	message _ self thisMessage.
	^ true! !

!OBKeyBindingsTest methodsFor: 'tests' stamp: 'cwp 10/31/2004 00:33'!
testBrowseIt
	self assert: self ksCmdB sends: #browseIt:! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:08'!
ksCmdN
	^ self modifier: 64 keycode: 110! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:09'!
ksCmdShiftE
	^ self modifier: 72 keycode: 69
! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:08'!
ksCmdM
	^ self modifier: 64 keycode: 109! !

!OBKeyBindingsTest methodsFor: 'asserting' stamp: 'cwp 10/31/2004 00:36'!
assert: evt sends: aSelector
	ptm keyStroke: evt.
	self assert: message notNil.
	self assert: message selector = aSelector.
	self assert: message arguments size = 1.
	self assert: message arguments first = #OBBrowser.! !

!OBKeyBindingsTest methodsFor: 'keystrokes' stamp: 'cwp 10/31/2004 01:09'!
ksCmdShiftN
	^ self modifier: 72 keycode: 78! !

!OBKeyBindingsTest methodsFor: 'running' stamp: 'cwp 10/30/2004 22:05'!
setUp
	ptm := (OBPluggableTextMorph
			on: self
			text: #text
			accept: #accept:notifying:
			readSelection: #selection
			menu: #menu:shifted:)
				font: Preferences standardCodeFont;
				yourself! !

!OBKeyBindingsTest methodsFor: 'callbacks' stamp: 'cwp 10/31/2004 00:49'!
referencesToIt: aSelector
	message := self thisMessage.
	^ true! !

TestCase subclass: #OBProtocolMockTest
	instanceVariableNames: 'mock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBProtocolMockTest methodsFor: 'asserting' stamp: 'cwp 2/13/2004 01:17'!
shouldFail: aBlock
	self should: aBlock raise: TestResult failure
				
					! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:18'!
testSelectorFailure
	mock expect: [:stub | stub foo].
	self shouldFail: [mock examine: [:stub | stub bar]]
	! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/29/2004 16:05'!
testTooManyExceptionsFailure
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true].
	self should:  [mock examine: [:stub | stub truth: Notification signal. Notification signal]]
		raise: Exception! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:20'!
testResume
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true];
		examine: [:stub | stub truth: Notification signal]! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:44'!
testNoExceptionFailure
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true].
	self shouldFail: [mock examine: [:stub | stub truth: true]] ! !

!OBProtocolMockTest methodsFor: 'running' stamp: 'cwp 2/13/2004 01:15'!
setUp
	mock _ OBProtocolMock new.! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:17'!
testExtraFailure
	mock expect: [:stub | stub foo].
	self shouldFail: [mock examine: [:stub | stub foo; bar]]! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 17:17'!
testAnswers
	| one two three |
	mock 
		expect: [:stub | stub int; int; int];
		answer: #(1 2 3);
		examine: [:stub | 	one _ stub int.
						two _ stub int.
						three _ stub int].
		self assert: one = 1.
		self assert: two = 2.
		self assert: three = 3! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 3/14/2004 15:39'!
testAnswersRecorder
	| one two three |
	mock 
		expect: [:stub | stub int; int; int; foo];
		answer: #(1 2);
		examine: [:stub | one _ stub int.
						two _ stub int.
						three _ stub int.
						three foo].
		self assert: one = 1.
		self assert: two = 2.! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:18'!
testMissingFailure
	mock expect: [:stub | stub foo; bar].
	self shouldFail: [mock examine: [:stub | stub foo]]! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:17'!
testArgFailure
	mock expect: [:stub | stub foo: #bar].
	self shouldFail: [mock examine: [:stub | stub foo: #baz]]
					! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/13/2004 01:18'!
testSuccess
	mock expect: [:stub | stub foo; foo: #bar].
	mock examine: [:stub | stub foo; foo: #bar]
				
					! !

!OBProtocolMockTest methodsFor: 'testing' stamp: 'cwp 2/28/2004 22:09'!
testWrongExceptionFailure
	mock 
		resume: Notification with: true;
		expect: [:stub | stub truth: true].
	self should: [mock examine: [:stub | stub truth: Error signal]] 
		raise: Error! !

PluggableTextMorph subclass: #OBPluggableTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBPluggableTextMorph methodsFor: 'as yet unclassified' stamp: 'cwp 10/30/2004 22:57'!
setText: aText
	scrollBar setValue: 0.0.
	textMorph
		ifNil: [textMorph _ OBTextMorph new
						contents: aText wrappedTo: self innerBounds width-6.
				textMorph setEditView: self.
				scroller addMorph: textMorph]
		ifNotNil: [textMorph newContents: aText].
	self hasUnacceptedEdits: false.
	self setScrollDeltas.! !

!OBPluggableTextMorph methodsFor: 'as yet unclassified' stamp: 'cwp 10/30/2004 23:07'!
keyStroke: evt
	^ textMorph keyStroke: evt! !

ProtoObject subclass: #OBMessageRecorder
	instanceVariableNames: 'mock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Tests'!

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 2/29/2004 15:52'!
asExplorerString
	^ 'an OBMessageRecorder'! !

!OBMessageRecorder methodsFor: 'initializing' stamp: 'cwp 2/13/2004 00:24'!
obmrSetMock: aMockObject
	mock _ aMockObject! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 2/29/2004 15:53'!
hasContentsInExplorer
	^ false! !

!OBMessageRecorder methodsFor: 'recording' stamp: 'cwp 2/28/2004 17:23'!
doesNotUnderstand: aMessage
	^ mock message: aMessage! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 3/17/2004 00:00'!
isMorph
	^ false! !

!OBMessageRecorder class methodsFor: 'as yet unclassified' stamp: 'cwp 2/13/2004 00:25'!
forMock: aMock
	^ self new obmrSetMock: aMock! !

!OBMessageRecorder methodsFor: 'debugging' stamp: 'cwp 2/13/2004 01:14'!
inspectorClass
	^ Inspector! !

MessageSend subclass: #OBAction
	instanceVariableNames: 'label buttonLabel monitor keystroke icon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBAction commentStamp: 'cwp 3/4/2004 20:52' prior: 0!
Actions represent commands for manipulating the graph domain. They can be made available through menus or buttons in the browser. They carry information on how they should be presented to the user, and are responsible for handling exceptions generated when they are triggered. Actions are created by Actors.

iVars: 

label 		a string which describes the action for the user.
monitor  	when the action is triggered, any notifications raised 
			will be passed to this object for processing!

!OBAction class methodsFor: 'instance creation' stamp: 'cwp 3/10/2004 23:36'!
label: aString buttonLabel: aString2 receiver: anObject selector: aSelector arguments: anArray
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		buttonLabel: aString2! !

!OBAction methodsFor: 'evaluating' stamp: 'cwp 3/14/2004 13:35'!
trigger
	[self value]
		on: OBNotification
		do: [:aNotification | 
			monitor notification: aNotification. 
			aNotification resume].! !

!OBAction class methodsFor: 'instance creation' stamp: 'hpt 5/17/2004 14:16'!
label: aString receiver: anObject selector: aSelector arguments: anArray keystroke: aChar
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		keystroke: aChar! !

!OBAction class methodsFor: 'instance creation' stamp: 'cwp 1/7/2004 09:20'!
label: aString receiver: anObject selector: aSelector arguments: anArray
	^ (self receiver: anObject selector: aSelector arguments: anArray) label: aString! !

!OBAction methodsFor: 'accessing' stamp: 'hpt 5/17/2004 14:16'!
keystroke
	^keystroke! !

!OBAction methodsFor: 'morphic' stamp: 'cwp 3/11/2004 22:58'!
button
	^ (PluggableButtonMorph
		on: self
		getState: nil
		action: #trigger
		label: #buttonLabelMorph)	
			onColor: Color lightGray lighter offColor: Color lightGray twiceLighter;
			borderWidth: 2;
			borderRaised;
			hResizing: #spaceFill;
			vResizing: #spaceFill;
			setBalloonText: label! !

!OBAction methodsFor: 'evaluating' stamp: 'cwp 1/7/2004 18:22'!
perform: aSelector orSendTo: anObject
	^ self perform: aSelector! !

!OBAction class methodsFor: 'instance creation' stamp: 'hpt 5/17/2004 16:50'!
label: aString receiver: anObject selector: aSelector arguments: anArray keystroke: aChar icon: anIcon
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		keystroke: aChar;
		icon: anIcon! !

!OBAction methodsFor: 'accessing' stamp: 'hpt 5/17/2004 16:42'!
icon: anIcon
	icon _ anIcon! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 3/10/2004 23:40'!
wantsButton
	^ buttonLabel notNil! !

!OBAction methodsFor: 'morphic' stamp: 'cwp 3/11/2004 22:59'!
buttonLabelMorph
	^ StringMorph 
		contents: self buttonLabel 
		font: Preferences standardButtonFont! !

!OBAction methodsFor: 'morphic' stamp: 'hpt 5/17/2004 17:00'!
addItemToMenu: aMenu
	aMenu
		add: self labelWithKeystroke
		target: self 
		selector: #trigger.
	Preferences menuWithIcons & self icon notNil
		ifTrue: [aMenu lastItem icon: self icon].
	aMenu addBlankIconsIfNecessary: MenuIcons blankIcon.! !

!OBAction methodsFor: 'accessing' stamp: 'hpt 5/17/2004 14:16'!
keystroke: aChar
	keystroke _ aChar! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 1/7/2004 09:12'!
label: aString
	label _ aString! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 3/10/2004 23:35'!
buttonLabel: aString
	buttonLabel _ aString
	! !

!OBAction methodsFor: 'accessing' stamp: 'hpt 5/17/2004 16:42'!
icon
	^icon! !

!OBAction methodsFor: 'morphic' stamp: 'hpt 5/17/2004 14:17'!
labelWithKeystroke
	^keystroke
		ifNil: [label]
		ifNotNil: [label, ' (', keystroke asString, ')']! !

!OBAction class methodsFor: 'instance creation' stamp: 'hpt 5/17/2004 16:50'!
label: aString receiver: anObject selector: aSelector arguments: anArray icon: anIcon
	^ (self receiver: anObject selector: aSelector arguments: anArray) 
		label: aString;
		icon: anIcon! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 1/7/2004 09:59'!
monitor: anObject
	monitor _ anObject! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 3/10/2004 23:35'!
buttonLabel
	^ buttonLabel! !

!OBAction methodsFor: 'accessing' stamp: 'cwp 1/7/2004 09:11'!
label
	^ label! !

Notification subclass: #OBNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBNotification commentStamp: 'cwp 3/5/2004 12:33' prior: 0!
An abstract superclass for UI-related notifications. This is necessary to allow subclasses to be trapped by OBAction, while allowing other notifications to pass.!

OBNotification subclass: #OBNodeSelected
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBNodeSelected commentStamp: 'cwp 3/5/2004 12:32' prior: 0!
This notification is signaled to make the indicated node the current selection.

iVars:

node	- the node which should be selected!

!OBNodeSelected methodsFor: 'notifying' stamp: 'cwp 3/14/2004 17:02'!
notify: aMonitor
	aMonitor nodeSelected: node! !

!OBNodeSelected class methodsFor: 'as yet unclassified' stamp: 'cwp 1/29/2004 23:37'!
for: aNode
	^ self new node: aNode! !

!OBNodeSelected methodsFor: 'accessing' stamp: 'cwp 1/29/2004 23:37'!
node: aNode
	node _ aNode! !

!OBNodeSelected methodsFor: 'accessing' stamp: 'cwp 1/29/2004 23:37'!
node
	^ node! !

OBNotification subclass: #OBNodeDeleted
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBNodeDeleted commentStamp: 'cwp 3/5/2004 12:31' prior: 0!
This notification is signaled when an action deletes the currently selected node.!

!OBNodeDeleted methodsFor: 'accessing' stamp: 'cwp 3/14/2004 16:56'!
node: aNode
	node _ aNode! !

!OBNodeDeleted methodsFor: 'accessing' stamp: 'cwp 3/14/2004 16:56'!
node
	^ node! !

!OBNodeDeleted methodsFor: 'notifying' stamp: 'cwp 3/14/2004 17:05'!
notify: aMonitor
	aMonitor nodeDeleted: node! !

!OBNodeDeleted class methodsFor: 'instance creation' stamp: 'cwp 3/14/2004 16:55'!
for: aNode
	^ self new node: aNode! !

OBNotification subclass: #OBDefinitionChanged
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBDefinitionChanged methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:18'!
definition
	"Answer the value of definition"

	^ definition! !

!OBDefinitionChanged methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:28'!
definition: anObject
	"Set the value of definition"
	
	definition _ anObject! !

!OBDefinitionChanged methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:51'!
notify: aMontitor
	aMontitor definitionChanged: self definition! !

!OBDefinitionChanged class methodsFor: 'instance creation' stamp: 'cwp 9/17/2004 00:18'!
definition: aClassDefinition 
	^ (self new definition: aClassDefinition) signal! !

OBNotification subclass: #OBListChanged
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBListChanged commentStamp: 'cwp 3/5/2004 12:31' prior: 0!
This notification is raised when some action changes the list in the current column of the browser, but the selected node should not change.!

!OBListChanged methodsFor: 'as yet unclassified' stamp: 'cwp 1/30/2004 00:33'!
notify: aMonitor
	aMonitor listChanged! !

Notification subclass: #OBInteractionRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

OBInteractionRequest subclass: #OBBrowseRequest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBBrowseRequest methodsFor: 'testing' stamp: 'cwp 10/17/2004 19:32'!
isBrowseRequest
	^ true! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 21:27'!
defaultAction
	^ browser openInMorphic! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 18:45'!
browser
	^ browser! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 18:45'!
browser: aBrowser
	browser _ aBrowser! !

!OBBrowseRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2004 18:45'!
signal: aBrowser
	^ (self new browser: aBrowser) signal! !

OBInteractionRequest subclass: #OBConfirmationRequest
	instanceVariableNames: 'prompt confirm cancel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBConfirmationRequest commentStamp: 'cwp 3/5/2004 12:30' prior: 0!
This notification is used to ask the user to confirm some kind potentially dangerous operation. Its default action is to open a PopUpMenu.

iVars:

prompt		- a string describing the situation the user is asked to confirm
confirm		- a string describing the action that will be taken if the user confirms
cancel		- a string describing the action that will be taken if the user does not confirm!

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:12'!
prompt: prompt confirm: confirm
	^ self prompt: prompt confirm: confirm cancel: 'Cancel'! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:02'!
prompt: prompt confirm: confirm cancel: cancel
	^ (self newPrompt: prompt confirm: confirm cancel: cancel) signal! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 2/28/2004 12:11'!
cancel
	self resume: false! !

!OBConfirmationRequest methodsFor: 'user interaction' stamp: 'cwp 2/28/2004 12:08'!
defaultAction
	^ PopUpMenu confirm: prompt trueChoice: confirm falseChoice: cancel! !

!OBConfirmationRequest methodsFor: 'initalizing' stamp: 'cwp 2/28/2004 12:00'!
setPrompt: promptString confirm: confirmString cancel: cancelString
	prompt _ promptString.
	confirm _ confirmString.
	cancel _ cancelString! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:13'!
prompt: prompt
	^ self prompt: prompt confirm: 'Ok'! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:09'!
newPrompt: prompt confirm: confirm cancel: cancel
	^ self new setPrompt: prompt confirm: confirm cancel: cancel! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 2/28/2004 12:11'!
ok
	self resume: true! !

OBInteractionRequest subclass: #OBChoiceRequest
	instanceVariableNames: 'prompt labels values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBChoiceRequest commentStamp: 'cwp 3/5/2004 12:26' prior: 0!
This notification is used to ask the user for to choose from a list of alternatives. Its defaultAction is to open a PopUpMenu. Test cases an intercept the notification and respond programmatically.

prompt	- A string describing the choice the user is asked to make.
labels	- A list of strings describing the alternatives.
values	- When the user chooses an alternative, the corresponding item from this list is returned!

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 3/4/2004 02:05'!
labels: anArray
	^ (self new setPrompt: nil labels: anArray values: anArray) signal! !

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 3/4/2004 02:27'!
prompt: aString labels: labelArray values: valueArray
	^ (self new setPrompt: aString labels: labelArray values: valueArray) signal! !

!OBChoiceRequest methodsFor: 'initializing' stamp: 'cwp 3/4/2004 02:03'!
setPrompt: aString labels: labelArray values: valueArray
	prompt _ aString.
	labels _ labelArray.
	values _ valueArray! !

!OBChoiceRequest methodsFor: 'signaling' stamp: 'cwp 5/11/2004 11:51'!
defaultAction
	^ values 
		at: ((PopUpMenu labelArray: labels) startUpWithCaption: prompt)
		ifAbsent: [nil]! !

OBInteractionRequest subclass: #OBTextRequest
	instanceVariableNames: 'prompt template'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBTextRequest commentStamp: 'cwp 3/5/2004 12:35' prior: 0!
This notification is used to ask the user to supply a short piece of text. Its defaultAction is to open a FillInTheBlank.

iVars:

prompt		- a string describing the text the user is asked to supply
template	- a default reply !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:23'!
prompt: aString
	prompt _ aString! !

!OBTextRequest methodsFor: 'user interaction' stamp: 'cwp 2/2/2004 21:27'!
requestText
	^ FillInTheBlankMorph 
		request: prompt
		initialAnswer: template
		centerAt: Sensor cursorPoint
		inWorld: World
		onCancelReturn: nil
		acceptOnCR: true! !

!OBTextRequest methodsFor: 'user interaction' stamp: 'cwp 2/2/2004 21:27'!
defaultAction
	self resume: self requestText! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 20:59'!
template: aString
	template _ aString! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:24'!
prompt
	^ prompt! !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/5/2004 20:31'!
prompt: promptString template: templateString
	^ (self new prompt: promptString; template: templateString) signal! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:00'!
template
	^ template! !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/2/2004 21:30'!
prompt: aString
	^ self new prompt: aString; template: ''! !

!OBInteractionRequest methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2004 19:31'!
isBrowseRequest
	^ false! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/17/2004 22:14'!
starShortNameOf: aMethodCategoryName
	^ (aMethodCategoryName = self methodCategoryPrefix)
		ifTrue: ['']
		ifFalse: [aMethodCategoryName allButFirst: self methodCategoryPrefix size + 1]
! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/4/2004 00:10'!
longNameOf: shortName
	^ shortName isEmpty
		ifTrue: [self systemCategoryPrefix]
		ifFalse: [self systemCategoryPrefix, '-', shortName]
! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/17/2004 22:19'!
starLongNameOf: shortName
	^ shortName isEmpty
		ifTrue: [self methodCategoryPrefix]
		ifFalse: [self methodCategoryPrefix, '-', shortName]
! !

!PackageInfo methodsFor: '*omnibrowser' stamp: 'cwp 3/4/2004 00:08'!
shortNameOf: aSystemCategoryName
	^ (aSystemCategoryName = self systemCategoryPrefix)
		ifTrue: ['']
		ifFalse: [aSystemCategoryName allButFirst: self systemCategoryPrefix size + 1]
! !

!PluggableButtonMorph methodsFor: '*omnibrowser' stamp: 'cwp 2/24/2004 19:07'!
isOn
	^ self color = onColor! !

Slider subclass: #OBScrollBar
	instanceVariableNames: 'upButton downButton pagingArea scrollDelta pageDelta interval menuSelector timeOfMouseDown timeOfLastScroll nextPageDirection currentScrollDelay'
	classVariableNames: 'CachedImages UpArrow8Bit UpArrow'
	poolDictionaries: ''
	category: 'OmniBrowser-Morphic'!

!OBScrollBar commentStamp: 'cwp 3/5/2004 12:01' prior: 0!
This class is a relic. It was originally created to work around bugs in ScrollBar which prevented it from working correctly in horizontal orientation. At some point the bugs should be fixed, and ScrollBar should be used instead.!

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
extent: p
        p x > p y
                ifTrue: [super extent: (p max: 42@8)]
                ifFalse: [super extent: (p max: 8@42)]! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:07'!
totalSliderArea
	| upperBoundsButton |
	upperBoundsButton _ upButton.
	upButton bottom > upperBoundsButton bottom
		ifTrue: [upperBoundsButton _ upButton].
	^ bounds isWide
		ifTrue: [upperBoundsButton bounds topRight corner: downButton bounds bottomLeft]
		ifFalse: [upperBoundsButton bounds bottomLeft corner: downButton bounds topRight].
! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
alternativeScrollbarLook
	^Preferences alternativeScrollbarLook! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDown
	self flag: #obsolete.
	downButton eventHandler: nil.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollDownInit! !

!OBScrollBar methodsFor: 'scroll timing' stamp: 'cwp 11/23/2003 18:07'!
resetTimer
	timeOfMouseDown _ Time millisecondClockValue.
	timeOfLastScroll _ timeOfMouseDown - 1000 max: 0.
	nextPageDirection _ nil.
	currentScrollDelay _ nil! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
scrollDelta
	^ scrollDelta! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUpInit
	upButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollUp.
	self startStepping.! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:07'!
sliderThickness
	^ self roundedScrollbarLook ifTrue:[15] ifFalse:[super sliderThickness]! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializePagingArea
	pagingArea := RectangleMorph newBounds: self totalSliderArea
								color: (Color r: 0.6 g: 0.6 b: 0.8).
	pagingArea borderWidth: 0.
	pagingArea on: #mouseDown send: #scrollPageInit: to: self.
	pagingArea on: #mouseUp send: #finishedScrolling to: self.
	self addMorph: pagingArea.
	self roundedScrollbarLook 
		ifTrue:[pagingArea color: (Color gray: 0.9)].! !

!OBScrollBar methodsFor: 'testing' stamp: 'cwp 11/23/2003 18:06'!
stepTime
	^ currentScrollDelay ifNil: [300]! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollBarAction
	^self valueOfProperty: #scrollBarAction! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeDownButton
	downButton := RectangleMorph
		newBounds: (self innerBounds bottomRight - self buttonExtent extent: self buttonExtent)
		color: self thumbColor.
	downButton on: #mouseDown send: #scrollDownInit to: self.
	downButton on: #mouseUp send: #finishedScrolling to: self.
	downButton addMorphCentered: (ImageMorph new image: 
		(self 
			cachedImageAt: (bounds isWide ifTrue: ['right'] ifFalse: ['down']) 
			ifAbsentPut: [
				self upArrow8Bit
					rotateBy: (bounds isWide ifTrue: [#right] ifFalse: [#pi])
					centerAt: 0@0
			]
		)
	).
	self roundedScrollbarLook ifTrue:[
		downButton color: Color veryLightGray.
		downButton borderStyle: (BorderStyle complexRaised width: 3).
	] ifFalse:[
		downButton setBorderWidth: 1 borderColor: #raised.
	].
	self addMorph: downButton.
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
setNextDirectionFromEvent: event

	nextPageDirection _ bounds isWide ifTrue: [
		event cursorPoint x >= slider center x
	]
	ifFalse: [
		event cursorPoint y >= slider center y
	]

! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDownInit
	downButton borderInset.
	self resetTimer.
	self scrollBarAction: #doScrollDown.
	self startStepping.! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeEmbedded: aBool
	"aBool == true => inboard scrollbar
	aBool == false => flop-out scrollbar"
	self roundedScrollbarLook ifFalse:[^self].
	aBool ifTrue:[
		self borderStyle: (BorderStyle inset width: 2).
		self cornerStyle: #square.
	] ifFalse:[
		self borderStyle: (BorderStyle width: 1 color: Color black).
		self cornerStyle: #rounded.
	].
	self removeAllMorphs.
	self initializeSlider.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
sliderShadowColor
	^ self roundedScrollbarLook
		ifTrue: [self sliderColor darker]
		ifFalse: [super sliderShadowColor]
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollPageInit: evt
	self resetTimer.
	self setNextDirectionFromEvent: evt.
	self scrollBarAction: #doScrollByPage.
	self startStepping.! !

!OBScrollBar methodsFor: 'accessing' stamp: 'cwp 11/23/2003 18:06'!
cachedImageAt: aKey ifAbsentPut: aBlock

	CachedImages ifNil: [CachedImages _ Dictionary new].
	^CachedImages at: aKey ifAbsentPut: aBlock! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
interval: d
	"Supply an optional floating fraction so slider can expand to indicate range"
	interval _ d min: 1.0.
	self expandSlider.
	self computeSlider.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUp: count
	self setValue: (value - (scrollDelta * count) - 0.000001 max: 0.0)! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
sliderExtent
	"The sliderExtent is now stored in the slider itself, not hardcoded as it is in the superclass."
	^slider extent! !

!OBScrollBar methodsFor: 'initialization' stamp: 'cwp 11/23/2003 18:07'!
initialize
	super initialize.
	scrollDelta _ 0.02.
	pageDelta _ 0.2.
	self roundedScrollbarLook ifTrue:[
		self borderStyle: ((BorderStyle complexFramed width: 2) "baseColor: Color gray")].! !

!OBScrollBar methodsFor: 'testing' stamp: 'cwp 11/23/2003 18:06'!
wantsSteps
	^self scrollBarAction notNil! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollUp
	self flag: #obsolete.
	upButton eventHandler: nil.
	upButton on: #mouseDown send: #scrollUpInit to: self.
	upButton on: #mouseUp send: #finishedScrolling to: self.
	^self scrollUpInit! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollDown
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value + scrollDelta + 0.000001 min: 1.0)! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollByPage
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 300 delay2: 100) ifFalse: [^ self].
	nextPageDirection
		ifTrue: [self setValue: (value + pageDelta min: 1.0)]
		ifFalse: [self setValue: (value - pageDelta max: 0.0)]
! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
finishedScrolling
	self stopStepping.
	self scrollBarAction: nil.
	self roundedScrollbarLook ifTrue:[
		upButton borderStyle: (BorderStyle complexRaised width: upButton borderWidth).
		downButton borderStyle: (BorderStyle complexRaised width: downButton borderWidth).
	] ifFalse:[
		downButton borderRaised.
		upButton borderRaised.
	].

! !

!OBScrollBar class methodsFor: 'as yet unclassified' stamp: 'cwp 11/23/2003 18:07'!
alwaysShowFlatScrollbarForAlternativeLook
	"Set this value to true, if you want to see the flat scrollbar look in flop-out mode as well as inboard.  Otherwise the flop-out scrollbar will be rounded and inboard will be flat."
	^ false! !

!OBScrollBar methodsFor: 'other events' stamp: 'cwp 11/23/2003 18:06'!
mouseDownInSlider: event
	interval = 1.0 ifTrue:
		["make the entire scrollable area visible if a full scrollbar is clicked on"
		self setValue: 0.
		self model hideOrShowScrollBar].
	super mouseDownInSlider: event! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeSlider
	self roundedScrollbarLook ifTrue:[
		self 
			initializeUpButton;
			initializeDownButton;
			initializePagingArea.
	] ifFalse:[
		self initializeUpButton; 
			initializeDownButton; 
			initializePagingArea.
	].
	super initializeSlider.
	self roundedScrollbarLook ifTrue:[
		slider cornerStyle: #rounded.
		slider borderStyle: (BorderStyle complexRaised width: 3).
		sliderShadow cornerStyle: #rounded.
	].
	self sliderColor: self sliderColor.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
scrollDelta: d1 pageDelta: d2
	"Supply optional increments for better scrolling of, eg, text"
	scrollDelta _ d1.
	pageDelta _ d2.! !

!OBScrollBar methodsFor: 'accessing' stamp: 'cwp 11/23/2003 18:06'!
adoptPaneColor: aColor
	"Adopt the given pane color"
	self alternativeScrollbarLook ifFalse:[^self].
	aColor ifNil:[^self].
	self sliderColor: aColor.! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollDown: count
	self setValue: (value + (scrollDelta * count) + 0.000001 min: 1.0)! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
scrollBarAction: aSymbol
	self setProperty: #scrollBarAction toValue: aSymbol! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
initializeUpButton
	upButton := self roundedScrollbarLook 
				ifTrue:  
					[RectangleMorph 
						newBounds: (self innerBounds topLeft extent: self buttonExtent)]
				ifFalse: 
					[RectangleMorph 
						newBounds: ((self innerBounds topLeft) extent: self buttonExtent)].
	upButton color: self thumbColor.
	upButton 
		on: #mouseDown
		send: #scrollUpInit
		to: self.
	upButton 
		on: #mouseUp
		send: #finishedScrolling
		to: self.
	upButton 
		addMorphCentered: (ImageMorph new image: (self 
						cachedImageAt: (bounds isWide ifTrue: ['left'] ifFalse: ['up'])
						ifAbsentPut: 
							[bounds isWide 
								ifTrue: [self upArrow8Bit rotateBy: #left centerAt: 0 @ 0]
								ifFalse: [self upArrow8Bit]])).
	self roundedScrollbarLook 
		ifTrue: 
			[upButton color: Color veryLightGray.
			upButton borderStyle: (BorderStyle complexRaised width: 3)]
		ifFalse: [upButton setBorderWidth: 1 borderColor: #raised].
	self addMorph: upButton! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
expandSlider
	"Compute the new size of the slider (use the old sliderThickness as a minimum)."
	| r |
	r _ self totalSliderArea.
	slider extent: (bounds isWide
		ifTrue: [((r width * interval) asInteger max: self sliderThickness) @ slider height]
		ifFalse: [slider width @ ((r height * interval) asInteger max: self sliderThickness)])! !

!OBScrollBar methodsFor: 'scrolling' stamp: 'cwp 11/23/2003 18:06'!
doScrollUp
	"Scroll automatically while mouse is down"
	(self waitForDelay1: 200 delay2: 40) ifFalse: [^ self].
	self setValue: (value - scrollDelta - 0.000001 max: 0.0)! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
thumbColor
	"Problem: Part of the ScrollBar/Slider code uses 'slider' to mean the entire scrollbar/slider widget, and part of it uses 'slider' to mean only the draggable 'thumb'.  This should be cleaned up so that 'thumb' is used instead of 'slider' where appropriate.  For now, the meaning of thumbColor is clear, at least."

	^ self alternativeScrollbarLook 
		ifTrue: [self sliderColor alphaMixed: 0.7 with: (Color gray: 0.95)]
		ifFalse: [Color veryLightGray]! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
pagingArea
	^pagingArea! !

!OBScrollBar methodsFor: 'stepping and presenter' stamp: 'cwp 11/23/2003 18:07'!
step
	| action |
	action _ self scrollBarAction.
	action ifNotNil:[self perform: action].! !

!OBScrollBar methodsFor: 'scroll timing' stamp: 'cwp 11/23/2003 18:07'!
waitForDelay1: delay1 delay2: delay2 
	"Return true if an appropriate delay has passed since the last scroll operation.
	The delay decreases exponentially from delay1 to delay2."

	| now scrollDelay |
	timeOfLastScroll isNil ifTrue: [self resetTimer].	"Only needed for old instances"
	now := Time millisecondClockValue.
	(scrollDelay := currentScrollDelay) isNil 
		ifTrue: [scrollDelay := delay1	"initial delay"].
	currentScrollDelay := scrollDelay * 9 // 10 max: delay2.	"decrease the delay"
	timeOfLastScroll := now.
	^true! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
roundedScrollbarLook
	"Rounded look currently only shows up in flop-out mode"
	^Preferences alternativeScrollbarLook and: [Preferences inboardScrollbars not and: [self class alwaysShowFlatScrollbarForAlternativeLook not]]
! !

!OBScrollBar methodsFor: 'other events' stamp: 'cwp 11/23/2003 18:06'!
menuButtonMouseDown: event
	event hand showTemporaryCursor: nil.
	self use: menuSelector orMakeModelSelectorFor: 'MenuButtonPressed:'
		in: [:sel | menuSelector _ sel.  model perform: sel with: event]! !

!OBScrollBar methodsFor: 'geometry' stamp: 'cwp 11/23/2003 18:06'!
buttonExtent
	^ bounds isWide
		ifTrue: [11 @ self innerBounds height]
		ifFalse: [self innerBounds width @ 11]! !

!OBScrollBar methodsFor: 'initialize' stamp: 'cwp 11/23/2003 18:06'!
upArrow8Bit

	"convert to 8-bit and convert white to transparent to avoid gratuitous conversion every time we put one in an ImageMorph"

	^UpArrow8Bit ifNil: [
		UpArrow8Bit _ (ColorForm mappingWhiteToTransparentFrom: UpArrow) asFormOfDepth: 8
	]! !

!OBScrollBar class methodsFor: 'class initialization' stamp: 'cwp 11/23/2003 18:07'!
initialize "ScrollBar initialize"

	UpArrow _ Form
		extent: 6@3
		fromArray: #(2r11e28 2r1111e27 2r111111e26)
		offset: 0@0.! !

!OBScrollBar methodsFor: 'access' stamp: 'cwp 11/23/2003 18:06'!
sliderColor: aColor
	"Change the color of the scrollbar to go with aColor."
	| buttonColor |
	super sliderColor: aColor.
	buttonColor _ self thumbColor.
	upButton color: buttonColor.
	downButton color: buttonColor.
	slider color: buttonColor.
	self alternativeScrollbarLook ifTrue:
		[self roundedScrollbarLook
			ifTrue:
				[self color: Color transparent.
				pagingArea color: aColor muchLighter.
				self borderStyle style == #simple 
					ifTrue:[self borderColor: aColor darker darker]
					ifFalse:[self borderStyle baseColor: aColor]]
			ifFalse:
				[pagingArea color: (aColor alphaMixed: 0.3 with: Color white).
				self borderWidth: 0]]
! !

!OBScrollBar methodsFor: 'model access' stamp: 'cwp 11/23/2003 18:06'!
setValue: newValue
	"Using roundTo: instead of truncateTo: ensures that scrollUp will scroll the same distance as scrollDown."
	^ super setValue: (newValue roundTo: scrollDelta)! !
OBScrollBar initialize!
